//////////////////////////////////////////////////////////////////////////////// 
// Copyright (c) 2006 Xilinx, Inc. 
// All Rights Reserved 
//////////////////////////////////////////////////////////////////////////////// 
//   ____  ____ 
//  /   /\/   / 
// /___/  \  /         Vendor: Xilinx 
// \   \   \/          Version: 1.0
//  \   \              Filename: control_block.v 
//  /   /              Date Last Modified: 4/31/2006
// /___/   /\          Date Created: 4/25/2006  
// \   \  /  \ 
//  \___\/\___\ 
// 
//Design Name: control_block 
//Purpose:  This module is the controls the address pointer of the RAID6 ip 
//          it can be replaced by a GPIO port of either PPC405 or microblaze
//          
//          This is a migration block of XAPP 731 for Virtex-5
//
////////////////////////////////////////////////////////////////////////////////
// 
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//
////////////////////////////////////////////////////////////////////////////////

`timescale 1ns / 1ps

module control_block(
   input  wire        clk,
   input  wire        rst,
   input  wire        rnw,
   input  wire        start,
   input  wire        mem_complete,
   input  wire        raid_complete,
   output reg         complete,//tells test block to check data
   output reg   [4:0] addr,
   output reg         wen,
   output reg         ren,
   input  wire [31:0] datain,
   output reg  [31:0] dataout,
   //input  wire [31:0] lost,
   //input  wire [31:0] mode,
   input  wire [31:0] size
   );


parameter IDLE       = 6'b000000;
parameter W0         = 6'b000001;
parameter W1         = 6'b000010;
parameter W2         = 6'b000011;
parameter W3         = 6'b000100;
parameter W4         = 6'b000101;
parameter W5         = 6'b000110;
parameter W6         = 6'b000111;
parameter W7         = 6'b001000;
parameter W8         = 6'b001001;
parameter W9         = 6'b001010;
parameter DONE_Q     = 6'b001011;
parameter W10        = 6'b001100;
parameter W11        = 6'b001101;
parameter DONE_P     = 6'b001110;
parameter W12        = 6'b001111;
parameter W13        = 6'b010000;
parameter DONE_D0    = 6'b010001;
parameter W14        = 6'b010010;
parameter W15        = 6'b010011;
parameter DONE_D1    = 6'b010100;
parameter W16        = 6'b010101;
parameter W17        = 6'b010110;
parameter DONE_D2    = 6'b010111;
parameter W18        = 6'b011000;
parameter W19        = 6'b011001;
parameter DONE_P_Q   = 6'b011010;
parameter W20        = 6'b011011;
parameter W21        = 6'b011100;
parameter DONE_D0_P  = 6'b011101;
parameter W22        = 6'b011110;
parameter W23        = 6'b011111;
parameter DONE_D1_P  = 6'b100000;
parameter W24        = 6'b100001;
parameter W25        = 6'b100010;
parameter DONE_D2_P  = 6'b100011;
parameter W26        = 6'b100100;
parameter W27        = 6'b100101;
parameter DONE_D0_Q  = 6'b100110;
parameter W28        = 6'b100111;
parameter W29        = 6'b101000;
parameter DONE_D1_Q  = 6'b101001;
parameter W30        = 6'b101010;
parameter W31        = 6'b101011;
parameter DONE_D2_Q  = 6'b101100;
parameter W32        = 6'b101101;
parameter W33        = 6'b101110;
parameter DONE_D0_D1 = 6'b101111;
parameter W34        = 6'b110000;
parameter W35        = 6'b110001;
parameter DONE_D1_D2 = 6'b110010;
parameter W36        = 6'b110011;
parameter W37        = 6'b110100;
parameter DONE_D2_D0 = 6'b110101;
parameter W38        = 6'b110110;
parameter W39        = 6'b110111;




parameter DONE = 6'b111111;


//defualt starting addr pointer
parameter D1   = 32'h00000000;
parameter D2   = 32'h00000500;
parameter D3   = 32'h00001000;
parameter D4   = 32'h00001500;
parameter P    = 32'h00002000;
parameter Q    = 32'h00002500;


	
reg  [5:0] state;

always @ (posedge clk or posedge rst)
begin
  if(rst)
  begin
    state    <= IDLE;
    complete <= 1'b0;
    addr     <= 5'b00001;
    wen      <= 1'b0;
    ren      <= 1'b0;
    dataout  <= 32'h0000;
  end
  else
  begin
    case (state)
      IDLE:  begin
               if(mem_complete)
               begin
                 state   <= W0;
                 wen     <= 1'b1;
                 dataout <= 32'h0002;//Gen P
               end
               else
               begin
                 state   <= IDLE;
               end
             end
      W0:    begin
               state   <= W1;
               addr    <= 5'b00010;
               dataout <= D1;
             end
      W1:    begin
               state   <= W2;
               addr    <= 5'b00011;
               dataout <= D2; 
             end
      W2:    begin
               state   <= W3;
               addr    <= 5'b00100;
               dataout <= D3; 
             end
      W3:    begin
               state   <= W4;
               addr    <= 5'b00101;
               dataout <= D4; 
             end
      W4:    begin
               state   <= W5;
               addr    <= 5'b00110;
               dataout <= P;
             end
      W5:    begin
               state   <= W6;
               addr    <= 5'b00111;
               dataout <= Q;
             end
      W6:    begin
               state   <= W7;
               addr    <= 5'b01000;
               dataout <= 32'h0001; //single P regen
             end
      W7:    begin
               state   <= W8;
               addr    <= 5'b01011;
               dataout <= 32'h00000001;
             end
      W8:    begin
               state    <= W9;
               addr     <= 5'b00000;
               //wen      <= 1'b1;
               dataout <= 32'h0001;
             end
      W9:    begin
               state    <= DONE_P;
               wen      <= 1'b0;
             end
   DONE_P:   begin
               if(raid_complete)
               begin
                 state    <= W10;
                 addr     <= 5'b00001;
                 wen      <= 1'b1;
                 dataout <= 32'h0001;//Gen Q
               end
               else
               begin
                 state    <= DONE_P;
                 wen      <= 1'b0;
               end
             end
     W10:    begin
               state   <= W11;
               addr    <= 5'b01000;
               dataout <= 32'h0002; //single Q regen
             end
     W11:    begin
               state   <= DONE_Q;
               addr    <= 5'b00000;
               dataout <= 32'h0001;
             end
   DONE_Q:   begin
               if(raid_complete)
               begin
                 state    <= W12;
                 addr     <= 5'b00001;
                 wen      <= 1'b1;
                 dataout <= 32'h0004;//D0
               end
               else
               begin
                 state    <= DONE_Q;
                 wen      <= 1'b0;
               end
             end
     W12:    begin
               state   <= W13;
               addr    <= 5'b01000;
               dataout <= 32'h0000; //single D regen
             end
     W13:    begin
               state   <= DONE_D0;
               addr    <= 5'b00000;
               dataout <= 32'h0001;
             end
  DONE_D0:   begin
               if(raid_complete)
               begin
                 state    <= W14;
                 addr     <= 5'b00001;
                 wen      <= 1'b1;
                 dataout <= 32'h0008;//D1
               end
               else
               begin
                 state    <= DONE_D0;
                 wen      <= 1'b0;
               end
             end    
     W14:    begin
               state   <= W15;
               addr    <= 5'b01000;
               dataout <= 32'h0000; //single D regen
             end
     W15:    begin
               state   <= DONE_D1;
               addr    <= 5'b00000;
               dataout <= 32'h0001;
             end

   DONE_D1:   begin
               if(raid_complete)
               begin
                 state    <= W16;
                 addr     <= 5'b00001;
                 wen      <= 1'b1;
                 dataout <= 32'h0010;//D2
               end
               else
               begin
                 state    <= DONE_D1;
                 wen      <= 1'b0;
               end
             end
     W16:    begin
               state   <= W17;
               addr    <= 5'b01000;
               dataout <= 32'h0000; //single D regen
             end
     W17:    begin
               state   <= DONE_D2;
               addr    <= 5'b00000;
               dataout <= 32'h0001;
             end

  DONE_D2:   begin
               if(raid_complete)
               begin
                 state    <= W18;
                 addr     <= 5'b00001;
                 wen      <= 1'b1;
                 dataout <= 32'h0003;//P & Q
               end
               else
               begin
                 state    <= DONE_D2;
                 wen      <= 1'b0;
               end
             end
     W18:    begin
               state   <= W19;
               addr    <= 5'b01000;
               dataout <= 32'h0007; //single P & Q regen
             end
     W19:    begin
               state   <= DONE_P_Q;
               addr    <= 5'b00000;
               dataout <= 32'h0001;
             end

 DONE_P_Q:   begin
               if(raid_complete)
               begin
                 state    <= W20;
                 addr     <= 5'b00001;
                 wen      <= 1'b1;
                 dataout <= 32'h0006;//D0 & P
               end
               else
               begin
                 state    <= DONE_P_Q;
                 wen      <= 1'b0;
               end
             end
     W20:    begin
               state   <= W21;
               addr    <= 5'b01000;
               dataout <= 32'h0006; //single D & P regen
             end
     W21:    begin
               state   <= DONE_D0_P;
               addr    <= 5'b00000;
               dataout <= 32'h0001;
             end

DONE_D0_P:   begin
               if(raid_complete)
               begin
                 state    <= W22;
                 addr     <= 5'b00001;
                 wen      <= 1'b1;
                 dataout <= 32'h000A;//D1 & P
               end
               else
               begin
                 state    <= DONE_D0_P;
                 wen      <= 1'b0;
               end
             end
     W22:    begin
               state   <= W23;
               addr    <= 5'b01000;
               dataout <= 32'h0006; //single D & P regen
             end
     W23:    begin
               state   <= DONE_D1_P;
               addr    <= 5'b00000;
               dataout <= 32'h0001;
             end

DONE_D1_P:   begin
               if(raid_complete)
               begin
                 state    <= W24;
                 addr     <= 5'b00001;
                 wen      <= 1'b1;
                 dataout <= 32'h0012;//D2 & P
               end
               else
               begin
                 state    <= DONE_D1_P;
                 wen      <= 1'b0;
               end
             end
     W24:    begin
               state   <= W25;
               addr    <= 5'b01000;
               dataout <= 32'h0006; //single D & P regen
             end
     W25:    begin
               state   <= DONE_D2_P;
               addr    <= 5'b00000;
               dataout <= 32'h0001;
             end

DONE_D2_P:   begin
               if(raid_complete)
               begin
                 state    <= W26;
                 addr     <= 5'b00001;
                 wen      <= 1'b1;
                 dataout <= 32'h0005;//D0 & Q
               end
               else
               begin
                 state    <= DONE_D2_P;
                 wen      <= 1'b0;
               end
             end
     W26:    begin
               state   <= W27;
               addr    <= 5'b01000;
               dataout <= 32'h0005; //single D & Q regen
             end
     W27:    begin
               state   <= DONE_D0_Q;
               addr    <= 5'b00000;
               dataout <= 32'h0001;
             end

DONE_D0_Q:   begin
               if(raid_complete)
               begin
                 state    <= W28;
                 addr     <= 5'b00001;
                 wen      <= 1'b1;
                 dataout <= 32'h0009;//D1 & Q
               end
               else
               begin
                 state    <= DONE_D0_Q;
                 wen      <= 1'b0;
               end
             end
     W28:    begin
               state   <= W29;
               addr    <= 5'b01000;
               dataout <= 32'h0005; //single D & Q regen
             end
     W29:    begin
               state   <= DONE_D1_Q;
               addr    <= 5'b00000;
               dataout <= 32'h001;
             end

DONE_D1_Q:   begin
               if(raid_complete)
               begin
                 state    <= W30;
                 addr     <= 5'b00001;
                 wen      <= 1'b1;
                 dataout <= 32'h0011;//D2 & Q
               end
               else
               begin
                 state    <= DONE_D1_Q;
                 wen      <= 1'b0;
               end
             end
     W30:    begin
               state   <= W31;
               addr    <= 5'b01000;
               dataout <= 32'h0005; //single D & Q regen
             end
     W31:    begin
               state   <= DONE_D2_Q;
               addr    <= 5'b00000;
               dataout <= 32'h0001;
             end

DONE_D2_Q:   begin
               if(raid_complete)
               begin
                 state    <= W32;
                 addr     <= 5'b00001;
                 wen      <= 1'b1;
                 dataout <= 32'h000C;//D0 & D1
               end
               else
               begin
                 state    <= DONE_D2_Q;
                 wen      <= 1'b0;
               end
             end
     W32:    begin
               state   <= W33;
               addr    <= 5'b01000;
               dataout <= 32'h0004; //single D & D regen
             end
     W33:    begin
               state   <= DONE_D0_D1;
               addr    <= 5'b00000;
               dataout <= 32'h0001;
             end

DONE_D0_D1:  begin
               if(raid_complete)
               begin
                 state    <= W34;
                 addr     <= 5'b00001;
                 wen      <= 1'b1;
                 dataout <= 32'h0018;//D1 & D2
               end
               else
               begin
                 state    <= DONE_D0_D1;
                 wen      <= 1'b0;
               end
             end
     W34:    begin
               state   <= W35;
               addr    <= 5'b01000;
               dataout <= 32'h0004; //single D & D regen
             end
     W35:    begin
               state   <= DONE_D1_D2;
               addr    <= 5'b00000;
               dataout <= 32'h0001;
             end

DONE_D1_D2:  begin
               if(raid_complete)
               begin
                 state    <= W36;
                 addr     <= 5'b00001;
                 wen      <= 1'b1;
                 dataout <= 32'h0014;//D2 & D0
               end
               else
               begin
                 state    <= DONE_D1_D2;
                 wen      <= 1'b0;
               end
             end
     W36:    begin
               state   <= W37;
               addr    <= 5'b01000;
               dataout <= 32'h0004; //single D & D regen
             end
     W37:    begin
               state   <= DONE_D2_D0;
               addr    <= 5'b00000;
               dataout <= 32'h0001;
             end

DONE_D2_D0:  begin
               if(raid_complete)
               begin
                 //state    <= W38;
                 //addr     <= 5'b00001;
                 //wen      <= 1'b1;
                 //dataout <= 32'h0000;
                
                 state    <= DONE;
                 complete <= 1'b1;
               end
               else
               begin
                 state    <= DONE_D2_D0;
                 wen      <= 1'b0;
               end
             end
      DONE:  begin
               state    <= DONE;
               complete <= 1'b0;
             end
    default: begin
               state <= IDLE;
               wen   <= 1'b0;
               ren   <= 1'b0;
               addr  <= 5'b00001;
               complete <= 1'b0;
               dataout  <= 32'h0000;
             end
	 endcase
  end
end





endmodule
