//////////////////////////////////////////////////////////////////////////////// 
// Copyright (c) 2006 Xilinx, Inc. 
// All Rights Reserved 
//////////////////////////////////////////////////////////////////////////////// 
//   ____  ____ 
//  /   /\/   / 
// /___/  \  /         Vendor: Xilinx 
// \   \   \/          Version: 1.0
//  \   \              Filename: hw_accel_top.v 
//  /   /              Date Last Modified: 4/31/2006
// /___/   /\          Date Created: 4/16/2006  
// \   \  /  \ 
//  \___\/\___\ 
// 
//Design Name: hw_accel_top 
//Purpose:  This module is the top module to instantiate RAID6 ip with a HW controller to 
//          replace the PPC405.  Also the MPMC is replaced with a 2:1 mux (for the 
//          controller and the RAID6 ip) to improve overall performance 
//          This is a migration of XAPP 731 for Virtex-5
//
////////////////////////////////////////////////////////////////////////////////
// 
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//
////////////////////////////////////////////////////////////////////////////////

`timescale 1ns / 1ps
`include "../rtl/raid6_parameters.v"

module hw_accel_top(
   input  wire       rst_n,
   output wire [7:0] status,
   input  wire       inject_n,

   inout      [63:0] DDR2_DQ,
   output     [13:0] DDR2_A,
   output     [1:0]  DDR2_BA,
   output            DDR2_RAS_N,
   output            DDR2_CAS_N,
   output            DDR2_WE_N,
   output            DDR2_RESET_N,
   output     [0:0]  DDR2_CS_N,
   output     [0:0]  DDR2_ODT,
   output     [0:0]  DDR2_CKE,
   output     [7:0]  DDR2_DM,
   input             SYS_CLK_P,
   input             SYS_CLK_N,
   input             CLK200_P,
   input             CLK200_N,
   input             SYS_RESET_IN,
   inout      [7:0]  DDR2_DQS,
   inout      [7:0]  DDR2_DQS_N,
   output     [1:0]  DDR2_CK,
   output     [1:0]  DDR2_CK_N

   );


wire  [7:0] control;	
wire        clk;
wire        SYS_BUF_CLK;
wire        rst;	
wire        start;
wire  [1:0] DCR_sel;	
wire [31:0] DCR_reg0;
wire [31:0] DCR_reg1;
wire [31:0] RAID_SIZE;
wire        RAID_done;

wire        RAID_clear;       
wire [31:0] RAID_lost;        
wire        RAID_reconstruct; 
wire [31:0] RAID_D1;         
wire [31:0] RAID_D2;          
wire [31:0] RAID_D3;         
wire [31:0] RAID_D4;          
wire [31:0] RAID_P;           
wire [31:0] RAID_Q;           
wire [31:0] RAID_M;  

wire [3:0] async_out;
wire       push_button_rst;
wire       rst_debounce;
wire       SYS_RST_FROM_MEM_CTRL;

wire [63:0] injected_error_data;

//inject error debounce circuit

wire inject_0;
reg  inject_1;
reg  inject_2;
reg  inject_3;
wire inject_debounce;
reg  inject_reg;
reg  inject_latch;

assign inject_debounce = (inject_1 & inject_2 & inject_3);
assign inject_0 = ~inject_n;  //reset is inverted in the memory controller

always @(posedge clk or posedge rst)
begin
  if(rst)
  begin
    inject_latch <= 1'b0;
    inject_reg   <= 1'b0;
  end
  else
  begin
    if(inject_debounce)
    begin
      if(inject_reg == 1'b1)
      begin
        inject_latch <= 1'b0;
      end
      else
      begin
        inject_reg  <= 1'b1;
        inject_latch <= 1'b1;
      end
    end
    else
    begin
      inject_latch <= inject_latch;
    end
  end
end


always @(posedge clk)
begin
 inject_1 <= inject_0;
 inject_2 <= inject_1;
 inject_3 <= inject_2;
end


//debounce circuitry
wire rst_0;
reg  rst_1;
reg  rst_2;
reg  rst_3;

assign rst_0 = rst_n;  //reset is inverted in the memory controller

always @(posedge SYS_BUF_CLK)
begin
 rst_1 <= rst_0;
 rst_2 <= rst_1;
 rst_3 <= rst_2;
end

assign rst_debounce = (rst_1 & rst_2 & rst_3);

assign push_button_rst = rst_debounce;


assign rst = (~rst_debounce || SYS_RST_FROM_MEM_CTRL);




//end debounce circuitry

wire         reg_start;
wire         reg_complete;

wire   [4:0] CTRL_addr;
wire         CTRL_wen;
wire         CTRL_ren;              
wire  [31:0] CTRL_datain;           
wire  [31:0] CTRL_dataout; 
wire  [31:0] lost;
wire  [31:0] mode;   


wire  [14:0] r6_cm_addr;
wire  [16:0] r6_cm_addr_unused;
wire  [14:0] tb_cm_addr;
wire  [16:0] tb_cm_addr_unused;
wire         r6_cm_en;
wire         tb_cm_en;
wire         r6_cm_wen;
wire         tb_cm_wen;
wire [`raid_width-1:0] r6_cm_data;
wire [`raid_width-1:0] tb_cm_data;
wire [`raid_width-1:0] cm_r6_data;
wire [`raid_width-1:0] cm_tb_data;

//parity bus
wire [15:0] r6_cm_parity;


//new for mig interface
wire        r6_mem_data_wen;
wire        r6_mem_addr_wen;
wire [35:0] r6_mem_addr;

wire        mem_init_done;

wire [`raid_width-1:0] WRITE_MEM_DATA;
wire [`raid_width-1:0] READ_MEM_DATA;
wire [15:0]  WRITE_MEM_PARITY_DATA;
wire [15:0]  READ_MEM_PARITY_DATA;
wire [35:0]  MEM_ADDR;
wire         MEM_SEL;
wire         CLK_0_FROM_MEM_CTRL_DCM;
wire         CLK_DV_FROM_MEM_CTRL_DCM;
wire         WRITE_DATA_FULL;
wire         ADDR_ALMOST_FULL;
wire         DATA_WREN;
wire         ADDR_WREN;


reg          raid_complete_reg;
wire         memory_write_addr_bit;

wire   [1:0] inject_error;
wire         RAID_SERR;
wire         RAID_DERR;
  
assign raid_clk              = CLK_DV_FROM_MEM_CTRL_DCM;
assign clk                   = CLK_0_FROM_MEM_CTRL_DCM;
assign memory_write_addr_bit = (~tb_cm_en & tb_cm_wen);
assign MEM_SEL               = (mem_init_done ^~ raid_complete_reg);


always @(posedge clk or posedge rst)
begin
  if(rst)
  begin
    raid_complete_reg <= 1'b0;
  end
  else
  begin
    if(reg_complete)
    begin
      raid_complete_reg <= 1'b1;
    end
    else
    begin
      raid_complete_reg <= raid_complete_reg;
    end 
  end
end

//instantiate RAID6 HW accelerator block
raid6_block r6_hwa( 
   .clk               (clk),
   .clk_mem           (clk), 
   .rst               (rst),
   .start             (RAID_reconstruct),
   .DCR_sel           (DCR_sel), 
   .DCR_reg0          (DCR_reg0), 
   .DCR_reg1          (DCR_reg1), 
   .RAID_SIZE         (RAID_SIZE), 
   .done              (RAID_done),
   .SINGLE_ERR        (RAID_SERR),
   .DOUBLE_ERR        (RAID_DERR),
   .gfc_done          (), 
   .mem_wrdata        (r6_cm_data[63:0]),    //raid_width
   .mem_wr_parity     (r6_cm_parity),
   .mem_rddata        (injected_error_data), //READ_MEM_DATA[63:0]), //raid_width
   .mem_rd_parity     (READ_MEM_PARITY_DATA[15:0]),
   .mem_addr          (r6_mem_addr),//36
   .mem_data_wen      (r6_mem_data_wen),
   .mem_addr_wen      (r6_mem_addr_wen),   
   .mem_data_valid    (READ_MEM_VALID) 
   );


//this is the injection error logic that would be used with the status/cntrl registers
//for simulation this is hooked up to the inject_n input implemented below

//assign injected_error_data[63:2] = READ_MEM_DATA[63:0] ; 
//assign injected_error_data[63:2] = inject_error[0] ? ( (~READ_MEM_DATA[0]) : READ_MEM_DATA[0] ; 
//assign injected_error_data[63:2] = inject_error[1] ? ((~READ_MEM_DATA[1]) : READ_MEM_DATA[1] ; 

//end status/cntrl inject error logic

assign injected_error_data = inject_latch ? ({READ_MEM_DATA[63:1], ~READ_MEM_DATA[0]}) : READ_MEM_DATA[63:0] ; 

//instantiate register mux to switch from controller to raid accessing the memory

genvar i;
   generate for(i = 0; i < `raid_width; i = i+32)
   begin: bus_mux_4_1_inst 

bus_mux_4_1 WRITE_MEM_DATA_mux_i(
   .bus_in0 (r6_cm_data[i+31:i]), 
   .bus_in1 (tb_cm_data[i+31:i]), 
   .bus_in2 (32'b0), 
   .bus_in3 (32'b0), 
   .sel     ({1'b0,MEM_SEL}), 
   .bus_out (WRITE_MEM_DATA[i+31:i])  
   );

   end  
 endgenerate

//these muxes are only for the parity bits
bus_mux_4_1 WRITE_MEM_PARITY_DATA_mux_0(
   .bus_in0 ({16'b0,r6_cm_parity[15:0]}),
   .bus_in1 (32'b0), 
   .bus_in2 (32'b0), 
   .bus_in3 (32'b0), 
   .sel     ({1'b0,MEM_SEL}), 
   .bus_out (WRITE_MEM_PARITY_DATA[15:0])  
   );




bus_mux_4_1 MEM_ADDR_mux_0(
   .bus_in0 ({17'b0,r6_mem_addr[31:0]}), 
   .bus_in1 ({17'b0,tb_cm_addr}), 
   .bus_in2 (32'b0), 
   .bus_in3 (32'b0), 
   .sel     ({1'b0,MEM_SEL}), 
   .bus_out (MEM_ADDR[31:0])  
   );

bus_mux_4_1 MEM_ADDR_control_mux_0(
   .bus_in0 ({17'b0,r6_mem_addr[35:32]}), 
   .bus_in1 ({17'b0,3'b000,memory_write_addr_bit }), 
   .bus_in2 (32'b0), 
   .bus_in3 (32'b0), 
   .sel     ({1'b0,MEM_SEL}), 
   .bus_out (MEM_ADDR[35:32])  
   );




bus_mux_4_1 CONTROL_mux_0(
   .bus_in0 ({r6_mem_data_wen,r6_mem_addr_wen}), 
   .bus_in1 ({tb_cm_en,tb_cm_wen}), 
   .bus_in2 (32'b0), 
   .bus_in3 (32'b0), 
   .sel     ({1'b0,MEM_SEL}), 
   .bus_out ({DATA_WREN,ADDR_WREN})  
   );




//instantitate DDR2 memory controller
//this reference design uses a 64bit DDR design, but the raid6 block
//only manipulates 32DDR bits(64bit SDR) to fit into the targeted device
mem r6_mem_cntrl(
.cntrl0_DDR2_DQ                      (DDR2_DQ),       //inout  [63:0]  
.cntrl0_DDR2_A                       (DDR2_A),        //output  [13:0]  
.cntrl0_DDR2_BA                      (DDR2_BA),       //output  [1:0]  
.cntrl0_DDR2_RAS_N                   (DDR2_RAS_N),    //output  
.cntrl0_DDR2_CAS_N                   (DDR2_CAS_N),    //output  
.cntrl0_DDR2_WE_N                    (DDR2_WE_N),     //output  
.cntrl0_DDR2_RESET_N                 (DDR2_RESET_N),  //output  
.cntrl0_DDR2_CS_N                    (DDR2_CS_N),     //output  [0:0]  
.cntrl0_DDR2_ODT                     (DDR2_ODT),      //output  [0:0]  
.cntrl0_DDR2_CKE                     (DDR2_CKE),      //output  [0:0]  
.cntrl0_DDR2_DM                      (DDR2_DM),       //output  [8:0]  
.SYS_CLK_P                           (SYS_CLK_P),     //input  
.SYS_CLK_N                           (SYS_CLK_N),        //input 
.SYS_BUF_CLK                         (SYS_BUF_CLK),         //output  
.CLK200_P                            (CLK200_P),               //input  
.CLK200_N                            (CLK200_N),                  //input  
.SYS_RESET_IN                        (push_button_rst ),             //input  
.SYS_RST_FROM_MEM_CTRL               (SYS_RST_FROM_MEM_CTRL),        //output 
.CLK_0_FROM_MEM_CTRL_DCM             (CLK_0_FROM_MEM_CTRL_DCM),      //output 
.CLK_DV_FROM_MEM_CTRL_DCM            (CLK_DV_FROM_MEM_CTRL_DCM),     //output 
.cntrl0_phy_init_initialization_done (phy_init_initialization_done), //output  
.cntrl0_WDF_ALMOST_FULL              (WRITE_DATA_FULL),              //output  
.cntrl0_AF_ALMOST_FULL               (ADDR_ALMOST_FULL),             //output  
.cntrl0_READ_DATA_VALID              (READ_MEM_VALID),               //output  
.cntrl0_APP_WDF_WREN                 (DATA_WREN),                    //input  
.cntrl0_APP_AF_WREN                  (ADDR_WREN),                    //input  
.cntrl0_BURST_LENGTH                 (),                               //output  [2:0]  
.cntrl0_APP_AF_ADDR                  (MEM_ADDR[35:0]),                    //input  [35:0]  
.cntrl0_READ_DATA_FIFO_OUT           ({READ_MEM_PARITY_DATA,READ_MEM_DATA}),                        //`memory_width  
.cntrl0_APP_WDF_DATA                 ({48'h0000000000000, WRITE_MEM_PARITY_DATA,WRITE_MEM_DATA}), //`memory_width  
.cntrl0_APP_MASK_DATA                (16'b0),                                  //input  [15:0]  
.cntrl0_DDR2_DQS                     (DDR2_DQS),                               //inout  [7:0]  
.cntrl0_DDR2_DQS_N                   (DDR2_DQS_N),                             //inout  [7:0]  
.cntrl0_DDR2_CK                      (DDR2_CK),                                //output  [1:0]  
.cntrl0_DDR2_CK_N                    (DDR2_CK_N)                               //output  [1:0]  
);



//instantiate registers
//these registers hold the status and information
//for generating or regenrating RAID6 data
//this block can be expanded to support more DISKs
//as can the RAID6 ip can as well
status_ctrl_reg r6_sc_reg(
   .clk              (clk),
   .rst              (rst),
   .addr             (CTRL_addr),  
   .wen              (CTRL_wen),
   .ren              (CTRL_ren),
   .datain           (CTRL_dataout), 
   .dataout          (CTRL_datain),
   .RAID_clear       (RAID_done),  
   .RAID_lost        (RAID_lost),  
   .RAID_reconstruct (RAID_reconstruct),  
   .RAID_D1          (RAID_D1),  
   .RAID_D2          (RAID_D2),  
   .RAID_D3          (RAID_D3),  
   .RAID_D4          (RAID_D4),  
   .RAID_P           (RAID_P),  
   .RAID_Q           (RAID_Q),  
   .RAID_M           (RAID_M), 
   .RAID_SIZE        (RAID_SIZE),
   .SBERR            (),
   .DBERR            (),
   .INJECT           (inject_error),
   .RAID_S_BIT_ERROR (RAID_SERR),
   .RAID_D_BIT_ERROR (RAID_DERR)

   );

//instantiate DCR register mux
bus_mux_4_1 r6_bus_mux_0(
   .bus_in0 (RAID_lost), 
   .bus_in1 (RAID_D1), 
   .bus_in2 (RAID_D2), 
   .bus_in3 (RAID_D3), 
   .sel     (DCR_sel), 
   .bus_out (DCR_reg0)  
   );

//instantiate DCR register mux
bus_mux_4_1 r6_bus_mux_1(
   .bus_in0 (RAID_M), 
   .bus_in1 (RAID_D4), 
   .bus_in2 (RAID_P), 
   .bus_in3 (RAID_Q), 
   .sel     (DCR_sel), 
   .bus_out (DCR_reg1)  
   );
	
//the blocks below emulated the system that controls the RAID6 IP
//these blocks would most likely be a processor that also controls 
//how data gets to/from the cache memory(DDR2 memory)

	
//instatntiate blocks setup registers for the RAID6 IP
control_block cb0(
   .clk           (clk),
   .rst           (rst),
   .rnw           (reg_rnw),
   .start         (reg_start),
   .mem_complete  (mem_init_done),
   .raid_complete (RAID_done),
   .complete      (reg_complete),
   .addr          (CTRL_addr),
   .wen           (CTRL_wen),
   .ren           (CTRL_ren),
   .datain        (CTRL_datain),
   .dataout       (CTRL_dataout),
   //.lost          (lost),
   //.mode          (mode),
   .size          (32'b0) //set to 512 blocks for now
   );


//instantiate block that generates and test the data in the cache memory	
test_block tb_0(
   .clk              (clk),
   .rst              (rst),
   .start            (reg_start),
   .complete         (reg_complete),
   .raid_done        (~RAID_reconstruct),
   //.lost             (lost),
   //.mode             (mode),
   .control          (control),
   .mem_ready        (phy_init_initialization_done),
   .status           (status),
   .mem_init_ready   (mem_init_done), 
   .mem_addr         ({tb_cm_addr_unused,tb_cm_addr}),
   .mem_datain       (tb_cm_data),    //output to cache memory
   .mem_wen          (tb_cm_wen),
   .mem_en           (tb_cm_en),
   .mem_dataout      (READ_MEM_DATA), //input from cache memory
   .mem_read_valid   (READ_MEM_VALID)
   );
	
endmodule

/*
	


//-----------------------------------------------------------------
  //
  //  ICON core wire declarations
  //
  //-----------------------------------------------------------------
  wire [35:0] control0;
  wire [35:0] control1;
  wire [63:0] data;
  wire [7:0] trig;
  wire [3:0] async_in;
 

assign data = ({MEM_ADDR[32], 2'b11, r6_mem_data_wen,r6_mem_addr_wen,2'b11,READ_MEM_DATA[31:0],r6_cm_data[15:0],MEM_ADDR[7:0]}); 

assign trig = ({MEM_ADDR[32],status[2:0],RAID_reconstruct,READ_MEM_VALID,r6_mem_data_wen,r6_mem_addr_wen});

//12:12:10
//000 = D1
//001 = D2
//010 = D3
//011 = D4
//100 = P
//101 = Q

assign async_in = status[3:0];

  //-----------------------------------------------------------------
  //
  //  ICON core instance
  //
  //-----------------------------------------------------------------
  icon i_icon
    (
      .control0(control0),
      .control1(control1)
    );

  //-----------------------------------------------------------------
  //
  //  ILA core instance
  //
  //-----------------------------------------------------------------
  ila i_ila
    (
      .control(control0),
      .clk(clk),
      .data(data),
      .trig0(trig)
    );
  //-----------------------------------------------------------------
  //
  //  VIO core instance
  //
  //-----------------------------------------------------------------
  vio i_vio
    (
      .control(control1),
      .async_in(async_in),
      .async_out(async_out)
    );





endmodule


//-------------------------------------------------------------------
//
//  ICON core module declaration
//
//-------------------------------------------------------------------
module icon 
  (
      control0,
      control1
  );
  output [35:0] control0;
  output [35:0] control1;
endmodule

//-------------------------------------------------------------------
//
//  ILA core module declaration
//
//-------------------------------------------------------------------
module ila
  (
    control,
    clk,
    data,
    trig0
  );
  input [35:0] control;
  input clk;
  input [63:0] data;
  input [7:0] trig0;
endmodule

//-------------------------------------------------------------------
//
//  VIO core module declaration
//
//-------------------------------------------------------------------
module vio
  (
    control,
    async_in,
    async_out
  );
  input  [35:0] control;
  input  [3:0] async_in;
  output [3:0] async_out;
endmodule
*/



