//////////////////////////////////////////////////////////////////////////////// 
// Copyright (c) 2006 Xilinx, Inc. 
// All Rights Reserved 
//////////////////////////////////////////////////////////////////////////////// 
//   ____  ____ 
//  /   /\/   / 
// /___/  \  /         Vendor: Xilinx 
// \   \   \/          Version: 1.0
//  \   \              Filename: status_ctrl_reg.v 
//  /   /              Date Last Modified: 4/31/2006
// /___/   /\          Date Created: 4/25/2006  
// \   \  /  \ 
//  \___\/\___\ 
// 
//Design Name: status_ctrl_reg 
//Purpose:  This module is contains the address pointers for the RAID6 ip 
//          it replaces the DCR_RAID6 block of XAPP 731 to allow for Microblaze 
//          or HW interface which is non DCR compliant (the old DCR block is still
//          available for those that request the DCR mode)
//          This is a migration of XAPP 731 for Virtex-5
//
////////////////////////////////////////////////////////////////////////////////
// 
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//
////////////////////////////////////////////////////////////////////////////////

`timescale 1ns / 1ps

module status_ctrl_reg(
   input  wire        clk,
   input  wire        rst,
   input  wire [4:0]  addr,
   input  wire        wen,
   input  wire        ren,
   input  wire [31:0] datain,
	output reg  [31:0] dataout,
   input  wire        RAID_clear,
   output reg  [31:0] RAID_lost,
	output reg         RAID_reconstruct,
   output reg  [31:0] RAID_D1,
   output reg  [31:0] RAID_D2,
   output reg  [31:0] RAID_D3,
   output reg  [31:0] RAID_D4,
   output reg  [31:0] RAID_P,
   output reg  [31:0] RAID_Q,
   output reg  [31:0] RAID_M,
   output reg  [31:0] RAID_SIZE,
   output reg         SBERR,
   output reg         DBERR,
   output reg   [1:0] INJECT,
   input  wire        RAID_S_BIT_ERROR,
   input  wire        RAID_D_BIT_ERROR
   );



      ////////////////////////////
     //Implement registers
     ////////////////////////////

     // RAID_reconstruct (0x0)
     always@( posedge clk or posedge rst)
          begin
          if ( rst ) begin
               RAID_reconstruct <= 1'b0;
          end
          else if (( addr == 5'b00000 ) && wen ) 
          begin
               RAID_reconstruct <= datain[0];
          end
          else if (RAID_clear)
          begin
               RAID_reconstruct <= 1'b0;
          end
          else if (( addr == 5'b00000 ) && ren ) 
          begin
               dataout[0] <= RAID_reconstruct;
					dataout[31:1] <= 30'b0;
          end
     end
          
     // RAID_lost (0x1)
     always@( posedge clk or posedge rst )
          begin
          if ( rst )
          begin
               RAID_lost <= 32'b0;
          end
          else if (( addr == 5'b00001 ) && wen )
          begin
               RAID_lost <= datain;
          end
     end

     // RAID_D1 (0x2)
     always@( posedge clk or posedge rst )
          begin
          if ( rst ) begin
               RAID_D1 <= 32'b0;
          end
          else if (( addr == 5'b00010 ) && wen )
          begin
               RAID_D1 <= datain;
          end
     end

     // RAID_D2 (0x3)
     always@( posedge clk or posedge rst )
          begin
          if ( rst ) begin
               RAID_D2 <= 32'b0;
          end
          else if (( addr == 5'b00011 ) && wen )
			 begin
               RAID_D2 <= datain;
          end
     end
      
     
     // RAID_D3 (0x4)
     always@( posedge clk or posedge rst )
          begin
          if ( rst ) begin
               RAID_D3 <= 32'b0;
          end
          else if (( addr == 5'b00100 ) && wen )
			 begin
               RAID_D3 <= datain;
          end
     end

     // RAID_D4 (0x5)
     always@( posedge clk or posedge rst )
          begin
          if ( rst ) begin
               RAID_D4 <= 32'b0;
          end
          else if (( addr == 5'b00101 ) && wen )
			 begin
               RAID_D4 <= datain;
          end
     end

    // RAID_P (0x6)
     always@( posedge clk or posedge rst )
          begin
          if ( rst ) begin
               RAID_P <= 32'b0;
          end
          else if (( addr == 5'b00110 ) && wen ) 
			 begin
               RAID_P <= datain;
          end
     end

    // RAID_Q (0x7)
     always@( posedge clk or posedge rst )
          begin
          if ( rst ) begin
               RAID_Q <= 32'b0;
          end
          else if (( addr == 5'b00111 ) && wen )
			 begin
               RAID_Q <= datain;
          end
     end

    // RAID_M (0x8)
     always@( posedge clk or posedge rst )
          begin
          if ( rst ) begin
               RAID_M <= 32'b0;
          end
          else if (( addr == 5'b01000 ) && wen )
			 begin
               RAID_M <= datain;
          end
     end

         //SBERR (0x9) //this indicates that there was an SBERR  
     always@( posedge clk or posedge rst )
          begin
          if ( rst ) begin
               SBERR <= 1'b0;
          end
          else if ( RAID_S_BIT_ERROR) begin
               SBERR <= 1'b1;
          end
          else if (( addr == 5'b01001 ) && wen )
	    begin
               DBERR <= 1'b0;
          end
     end

     //DBERR (0xA)//this indicates that there was a DBERR
     always@( posedge clk or posedge rst )
          begin
          if ( rst ) begin
               DBERR <= 1'b0;
         end          
         else if (( RAID_D_BIT_ERROR))
	   begin	
               DBERR <= 1'b1;
         end
         else if (( addr == 5'b01010 ) && wen )
	   begin
               DBERR <= 1'b0;
         end
     end
     // INJECT (0xB)  this causes an injected error
     always@( posedge clk or posedge rst )
          begin
          if ( rst ) begin
               INJECT <= 2'b00;
          end
          else if (( addr == 5'b01011 ) && wen )
	    begin
               INJECT <= datain[1:0];
          end
     end
     // RAID_SIZE (0xC)
     always@( posedge clk or posedge rst )
          begin
          if ( rst ) begin
               RAID_SIZE <= 32'b01;
          end
          else if (( addr == 5'b01100 ) && wen )
			 begin
               RAID_SIZE <= datain;
          end
     end

endmodule
