//////////////////////////////////////////////////////////////////////////////// 
// Copyright (c) 2006 Xilinx, Inc. 
// All Rights Reserved 
//////////////////////////////////////////////////////////////////////////////// 
//   ____  ____ 
//  /   /\/   / 
// /___/  \  /         Vendor: Xilinx 
// \   \   \/          Version: 1.0
//  \   \              Filename: test_block.v 
//  /   /              Date Last Modified: 6/6/2006
// /___/   /\          Date Created: 4/25/2006  
// \   \  /  \ 
//  \___\/\___\ 
// 
//Design Name: test_block 
//Purpose:  This module is the is a HW controller to 
//          replace the PPC405 in original XAPP731.   
//          It generates the data and checks the data for the different RAID6 scenarios
//          This is a migration of XAPP 731 for Virtex-5 LX/LXT devices
//
////////////////////////////////////////////////////////////////////////////////
// 
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//
////////////////////////////////////////////////////////////////////////////////
`timescale 1ns / 1ps
`include "../rtl/raid6_parameters.v"

module test_block(
   input  wire                     clk,
   input  wire                     rst,
   output reg                      start,
   input  wire                     complete,
   input  wire                     raid_done,
   //output reg               [31:0] lost,
   //output reg               [31:0] mode,
   input  wire               [7:0] control,
   input  wire                     mem_ready,
   output reg                [7:0] status,
   output reg                      mem_init_ready,
   output reg               [31:0] mem_addr,
   output reg  [`memory_width-1:0] mem_datain,
   output wire                     mem_wen,
   output wire                     mem_en,
   input  wire [`memory_width-1:0] mem_dataout,
   input  wire                     mem_read_valid
   );


//defualt starting addr pointer for the different blocks(P and Q won't be checked)
parameter D1   = 32'h00000000;
parameter D2   = 32'h00000500;
parameter D3   = 32'h00001000;
parameter D4   = 32'h00001500;
parameter P    = 32'h00002000;
parameter Q    = 32'h00002500;

parameter IDLE = 5'b00000;
parameter W0   = 5'b00001;
parameter W1   = 5'b00010;
parameter W2   = 5'b00011;
parameter W3   = 5'b00100;
parameter W4   = 5'b00101;
parameter W5   = 5'b00110;
parameter W6   = 5'b00111;
parameter W7   = 5'b01000;
parameter WAIT = 5'b11110;

parameter R0   = 5'b10000;
parameter R1   = 5'b10001;
parameter R2   = 5'b10010;
parameter R3   = 5'b10011;
parameter R4   = 5'b10100;
parameter R5   = 5'b10101;
parameter R6   = 5'b10110;
parameter R7   = 5'b10111;
parameter R8   = 5'b11000;
parameter R9   = 5'b11001;
parameter R10  = 5'b11010;
parameter R11  = 5'b11011;

parameter DONE = 5'b11111;

reg [4:0] state;
reg [3:0] mig_cnt;
reg       mem_addr_wen;
reg       mem_data_wen;
reg [7:0] status_reg;

assign mem_wen = mem_addr_wen;
assign mem_en  = mem_data_wen;

always @(posedge clk or posedge rst)
begin
  if (rst)
  begin
    status <= 8'h00;
  end
  else
  begin
    status <= status_reg;
  end
end



always @ (posedge clk or posedge rst)
begin
  if(rst)
  begin
     //lost        <= 32'h0000;//32
     //mode        <= 32'h0000;//32
     status_reg      <= 8'h00;
     mem_init_ready   <= 1'b0;
     mem_addr    <= D1;
     mem_datain[127:112] <= 16'h0000;
     mem_datain[111:96]  <= 16'h0000;
     mem_datain[95:80]   <= 16'h0000;
     mem_datain[79:64]   <= 16'h0000;
     mem_datain[63:48]   <= 16'h0000;
     mem_datain[47:32]   <= 16'h0000;
     mem_datain[31:16]   <= 16'h0000;
     mem_datain[15:0]    <= 16'h0000;
     mem_addr_wen     <= 1'b0;
     mem_data_wen     <= 1'b0;
     state       <= IDLE;
     mig_cnt     <= 4'b0001;
  end
  else
  begin
    case (state)
      IDLE:   begin
                if(mem_ready)
                begin
                  mem_addr_wen <= 1'b1;
                  mem_data_wen <= 1'b1;
                  state   <= W0;
                end
                else
                begin
                  state   <= IDLE;
                end						
              end
        W0:   begin
                mem_addr_wen <= 1'b0;
                //mem_datain <= mem_datain + 128'h00010001000100010001000100010001;
                  mem_datain[127:112] <= mem_datain[127:112] + 16'h0001;
                  mem_datain[111:96]  <= mem_datain[111:96]  + 16'h0001;
                  mem_datain[95:80]   <= mem_datain[95:80]   + 16'h0001;
                  mem_datain[79:64]   <= mem_datain[79:64]   + 16'h0001;
                  mem_datain[63:48]   <= mem_datain[63:48]   + 16'h0001;
                  mem_datain[47:32]   <= mem_datain[47:32]   + 16'h0001;
                  mem_datain[31:16]   <= mem_datain[31:16]   + 16'h0001;
                  mem_datain[15:0]    <= mem_datain[15:0]    + 16'h0001;
                state <= W1;   
              end
        W1:   begin
                if(mem_addr == (D1 + 32'h01fc))
                begin
                  state <= W2;
                  mem_addr_wen <= 1'b1;
                  mem_data_wen <= 1'b1;
                  mem_datain[127:112] <= 16'h0001;
                  mem_datain[111:96]  <= 16'h0001;
                  mem_datain[95:80]   <= 16'h0001;
                  mem_datain[79:64]   <= 16'h0001;
                  mem_datain[63:48]   <= 16'h0001;
                  mem_datain[47:32]   <= 16'h0001;
                  mem_datain[31:16]   <= 16'h0001;
                  mem_datain[15:0]    <= 16'h0001;
                  mem_addr      <= D2;
                end
                else
                begin
                  
                //mem_datain <= mem_datain + 128'h00010001000100010001000100010001;
                  mem_datain[127:112] <= mem_datain[127:112] + 16'h0001;
                  mem_datain[111:96]  <= mem_datain[111:96]  + 16'h0001;
                  mem_datain[95:80]   <= mem_datain[95:80]   + 16'h0001;
                  mem_datain[79:64]   <= mem_datain[79:64]   + 16'h0001;
                  mem_datain[63:48]   <= mem_datain[63:48]   + 16'h0001;
                  mem_datain[47:32]   <= mem_datain[47:32]   + 16'h0001;
                  mem_datain[31:16]   <= mem_datain[31:16]   + 16'h0001;
                  mem_datain[15:0]    <= mem_datain[15:0]    + 16'h0001;
                  mem_addr   <= mem_addr + 3'b100;
                  mem_addr_wen    <= 1'b1;
                  state <= W0;
                end
              end
        W2:   begin
                mem_addr_wen <= 1'b0;
                //mem_datain <= mem_datain + 128'h00010001000100010001000100010001;
                  mem_datain[127:112] <= mem_datain[127:112] + 16'h0001;
                  mem_datain[111:96]  <= mem_datain[111:96]  + 16'h0001;
                  mem_datain[95:80]   <= mem_datain[95:80]   + 16'h0001;
                  mem_datain[79:64]   <= mem_datain[79:64]   + 16'h0001;
                  mem_datain[63:48]   <= mem_datain[63:48]   + 16'h0001;
                  mem_datain[47:32]   <= mem_datain[47:32]   + 16'h0001;
                  mem_datain[31:16]   <= mem_datain[31:16]   + 16'h0001;
                  mem_datain[15:0]    <= mem_datain[15:0]    + 16'h0001;
                state <= W3;
              end
        W3:   begin
                if(mem_addr == (D2 + 32'h01fc))
                begin
                  state <= W4;
                  mem_addr_wen <= 1'b1;
                  mem_data_wen <= 1'b1;
                  mem_datain[127:112] <= 16'h0002;
                  mem_datain[111:96]  <= 16'h0002;
                  mem_datain[95:80]   <= 16'h0002;
                  mem_datain[79:64]   <= 16'h0002;
                  mem_datain[63:48]   <= 16'h0002;
                  mem_datain[47:32]   <= 16'h0002;
                  mem_datain[31:16]   <= 16'h0002;
                  mem_datain[15:0]    <= 16'h0002;
                  mem_addr      <= D3;
                end
                else
                begin
                //mem_datain <= mem_datain + 128'h00010001000100010001000100010001;
                  mem_datain[127:112] <= mem_datain[127:112] + 16'h0001;
                  mem_datain[111:96]  <= mem_datain[111:96]  + 16'h0001;
                  mem_datain[95:80]   <= mem_datain[95:80]   + 16'h0001;
                  mem_datain[79:64]   <= mem_datain[79:64]   + 16'h0001;
                  mem_datain[63:48]   <= mem_datain[63:48]   + 16'h0001;
                  mem_datain[47:32]   <= mem_datain[47:32]   + 16'h0001;
                  mem_datain[31:16]   <= mem_datain[31:16]   + 16'h0001;
                  mem_datain[15:0]    <= mem_datain[15:0]    + 16'h0001;
                  mem_addr   <= mem_addr + 3'b100;
                  mem_addr_wen    <= 1'b1;
                  state <= W2;
                end
              end
        W4:   begin
                mem_addr_wen <= 1'b0;
                  mem_datain[127:112] <= mem_datain[127:112] + 16'h0001;
                  mem_datain[111:96]  <= mem_datain[111:96]  + 16'h0001;
                  mem_datain[95:80]   <= mem_datain[95:80]   + 16'h0001;
                  mem_datain[79:64]   <= mem_datain[79:64]   + 16'h0001;
                  mem_datain[63:48]   <= mem_datain[63:48]   + 16'h0001;
                  mem_datain[47:32]   <= mem_datain[47:32]   + 16'h0001;
                  mem_datain[31:16]   <= mem_datain[31:16]   + 16'h0001;
                  mem_datain[15:0]    <= mem_datain[15:0]    + 16'h0001;
                state <= W5;
              end
        W5:   begin
                if(mem_addr == (D3 + 32'h01fc))
                begin
                  state <= WAIT;
                  mem_addr_wen <= 1'b0;
                  mem_data_wen <= 1'b1;
                  mem_init_ready   <= 1'b1;
                  //mem_datain    <= 128'h00020002000200020002000200020002;
                  //mem_addr      <= D2;
                end
                else
                begin

                //mem_datain <= mem_datain + 128'h00010001000100010001000100010001;
                  mem_datain[127:112] <= mem_datain[127:112] + 16'h0001;
                  mem_datain[111:96]  <= mem_datain[111:96]  + 16'h0001;
                  mem_datain[95:80]   <= mem_datain[95:80]   + 16'h0001;
                  mem_datain[79:64]   <= mem_datain[79:64]   + 16'h0001;
                  mem_datain[63:48]   <= mem_datain[63:48]   + 16'h0001;
                  mem_datain[47:32]   <= mem_datain[47:32]   + 16'h0001;
                  mem_datain[31:16]   <= mem_datain[31:16]   + 16'h0001;
                  mem_datain[15:0]    <= mem_datain[15:0]    + 16'h0001;
                  mem_addr   <= mem_addr + 3'b100;
                  mem_addr_wen    <= 1'b1;
                  state <= W4;
                end
              end


        WAIT: begin
                if(complete)
                begin
                  state      <= R0;
                       mem_datain[127:112] <= 16'h0000;
     mem_datain[111:96]  <= 16'h0000;
     mem_datain[95:80]   <= 16'h0000;
     mem_datain[79:64]   <= 16'h0000;
     mem_datain[63:48]   <= 16'h0000;
     mem_datain[47:32]   <= 16'h0000;
     mem_datain[31:16]   <= 16'h0000;
     mem_datain[15:0]    <= 16'h0000;
                  mem_addr_wen    <= 1'b0;
                  mem_data_wen    <= 1'b0;
                  mem_addr   <= D1;
                end
                else
                begin
                  state      <= WAIT;
                end
              end		
        R0:   begin
                state      <= R1;
                     mem_datain[127:112] <= 16'h0000;
     mem_datain[111:96]  <= 16'h0000;
     mem_datain[95:80]   <= 16'h0000;
     mem_datain[79:64]   <= 16'h0000;
     mem_datain[63:48]   <= 16'h0000;
     mem_datain[47:32]   <= 16'h0000;
     mem_datain[31:16]   <= 16'h0000;
     mem_datain[15:0]    <= 16'h0000;
                mem_addr_wen    <= 1'b1;
                mem_addr   <= D1;
              end		
        R1:   begin
                state      <= R2;
                mem_addr   <= mem_addr + 3'b100;
              end
        R2:   begin
                state      <= R3;
                mem_addr   <= mem_addr + 3'b100;
              end	
        R3:   begin
                if(mem_addr == (D1 + 32'h007c))
                begin
                  state      <= R4;
                  mem_addr_wen    <= 1'b0;
                  mem_addr   <= D2;
                  if (mem_read_valid)
                  begin
                    if(mem_datain[15:0] == 16'h003f)
                    begin
     mem_datain[127:112] <= 16'h0001;
     mem_datain[111:96]  <= 16'h0001;
     mem_datain[95:80]   <= 16'h0001;
     mem_datain[79:64]   <= 16'h0001;
     mem_datain[63:48]   <= 16'h0001;
     mem_datain[47:32]   <= 16'h0001;
     mem_datain[31:16]   <= 16'h0001;
     mem_datain[15:0]    <= 16'h0001;
                    end
                    else
                    begin
                //mem_datain <= mem_datain + 128'h00010001000100010001000100010001;
                  mem_datain[127:112] <= mem_datain[127:112] + 16'h0001;
                  mem_datain[111:96]  <= mem_datain[111:96]  + 16'h0001;
                  mem_datain[95:80]   <= mem_datain[95:80]   + 16'h0001;
                  mem_datain[79:64]   <= mem_datain[79:64]   + 16'h0001;
                  mem_datain[63:48]   <= mem_datain[63:48]   + 16'h0001;
                  mem_datain[47:32]   <= mem_datain[47:32]   + 16'h0001;
                  mem_datain[31:16]   <= mem_datain[31:16]   + 16'h0001;
                  mem_datain[15:0]    <= mem_datain[15:0]    + 16'h0001;
                    end
                    if(mem_datain[15:0] == mem_dataout[15:0])
                    begin
                      status_reg[0] <= 1'b1;
                    end
                    else
                    begin
                      status_reg[1] <= 1'b1;
                    end
                  end
                end
                else
                begin
                  mem_addr   <= mem_addr + 3'b100;
                  if (mem_read_valid)
                  begin                   
                    if(mem_datain[15:0] == 16'h003f)
                    begin
                           mem_datain[127:112] <= 16'h0001;
     mem_datain[111:96]  <= 16'h0001;
     mem_datain[95:80]   <= 16'h0001;
     mem_datain[79:64]   <= 16'h0001;
     mem_datain[63:48]   <= 16'h0001;
     mem_datain[47:32]   <= 16'h0001;
     mem_datain[31:16]   <= 16'h0001;
     mem_datain[15:0]    <= 16'h0001;
                    end
                    else
                    begin
                //mem_datain <= mem_datain + 128'h00010001000100010001000100010001;
                  mem_datain[127:112] <= mem_datain[127:112] + 16'h0001;
                  mem_datain[111:96]  <= mem_datain[111:96]  + 16'h0001;
                  mem_datain[95:80]   <= mem_datain[95:80]   + 16'h0001;
                  mem_datain[79:64]   <= mem_datain[79:64]   + 16'h0001;
                  mem_datain[63:48]   <= mem_datain[63:48]   + 16'h0001;
                  mem_datain[47:32]   <= mem_datain[47:32]   + 16'h0001;
                  mem_datain[31:16]   <= mem_datain[31:16]   + 16'h0001;
                  mem_datain[15:0]    <= mem_datain[15:0]    + 16'h0001;
                    end
                    if(mem_datain[15:0] == mem_dataout[15:0])
                    begin
                      status_reg[0] <= 1'b1;
                    end
                    else
                    begin
                      status_reg[1] <= 1'b1;
                    end
                  end

                end
              end	
        R4:   begin
                  if (mem_read_valid)
                  begin
                    if(mem_datain[15:0] == 16'h003f)
                    begin
                           mem_datain[127:112] <= 16'h0001;
     mem_datain[111:96]  <= 16'h0001;
     mem_datain[95:80]   <= 16'h0001;
     mem_datain[79:64]   <= 16'h0001;
     mem_datain[63:48]   <= 16'h0001;
     mem_datain[47:32]   <= 16'h0001;
     mem_datain[31:16]   <= 16'h0001;
     mem_datain[15:0]    <= 16'h0001;
                      state      <= R5;
                      mem_addr_wen    <= 1'b1;
                    end
                    else
                    begin
                //mem_datain <= mem_datain + 128'h00010001000100010001000100010001;
                  mem_datain[127:112] <= mem_datain[127:112] + 16'h0001;
                  mem_datain[111:96]  <= mem_datain[111:96]  + 16'h0001;
                  mem_datain[95:80]   <= mem_datain[95:80]   + 16'h0001;
                  mem_datain[79:64]   <= mem_datain[79:64]   + 16'h0001;
                  mem_datain[63:48]   <= mem_datain[63:48]   + 16'h0001;
                  mem_datain[47:32]   <= mem_datain[47:32]   + 16'h0001;
                  mem_datain[31:16]   <= mem_datain[31:16]   + 16'h0001;
                  mem_datain[15:0]    <= mem_datain[15:0]    + 16'h0001;
                    end
                    if(mem_datain[15:0] == mem_dataout[15:0])
                    begin
                      status_reg[0] <= 1'b1;
                    end
                    else
                    begin
                      status_reg[1] <= 1'b1;
                    end
                  end
              end		
        R5:   begin
                if(mem_addr == (D2 + 32'h007c))
                begin
                  state      <= R6;
                  mem_addr_wen    <= 1'b0;
                  mem_addr   <= D3;
                  if (mem_read_valid)
                  begin
                    if(mem_datain[15:0] == 16'h0040)
                    begin
                           mem_datain[127:112] <= 16'h0002;
     mem_datain[111:96]  <= 16'h0002;
     mem_datain[95:80]   <= 16'h0002;
     mem_datain[79:64]   <= 16'h0002;
     mem_datain[63:48]   <= 16'h0002;
     mem_datain[47:32]   <= 16'h0002;
     mem_datain[31:16]   <= 16'h0002;
     mem_datain[15:0]    <= 16'h0002;
                    end
                    else
                    begin
                //mem_datain <= mem_datain + 128'h00010001000100010001000100010001;
                  mem_datain[127:112] <= mem_datain[127:112] + 16'h0001;
                  mem_datain[111:96]  <= mem_datain[111:96]  + 16'h0001;
                  mem_datain[95:80]   <= mem_datain[95:80]   + 16'h0001;
                  mem_datain[79:64]   <= mem_datain[79:64]   + 16'h0001;
                  mem_datain[63:48]   <= mem_datain[63:48]   + 16'h0001;
                  mem_datain[47:32]   <= mem_datain[47:32]   + 16'h0001;
                  mem_datain[31:16]   <= mem_datain[31:16]   + 16'h0001;
                  mem_datain[15:0]    <= mem_datain[15:0]    + 16'h0001;
                    end
                    if(mem_datain[15:0] == mem_dataout[15:0])
                    begin
                      status_reg[0] <= 1'b1;
                    end
                    else
                    begin
                      status_reg[1] <= 1'b1;
                    end
                  end
                end
                else
                begin
                  mem_addr   <= mem_addr + 3'b100;
                  if (mem_read_valid)
                  begin
                    if(mem_datain[15:0] == 16'h0040)
                    begin
mem_datain[127:112] <= 16'h0002;
     mem_datain[111:96]  <= 16'h0002;
     mem_datain[95:80]   <= 16'h0002;
     mem_datain[79:64]   <= 16'h0002;
     mem_datain[63:48]   <= 16'h0002;
     mem_datain[47:32]   <= 16'h0002;
     mem_datain[31:16]   <= 16'h0002;
     mem_datain[15:0]    <= 16'h0002;
                    end
                    else
                    begin
                //mem_datain <= mem_datain + 128'h00010001000100010001000100010001;
                  mem_datain[127:112] <= mem_datain[127:112] + 16'h0001;
                  mem_datain[111:96]  <= mem_datain[111:96]  + 16'h0001;
                  mem_datain[95:80]   <= mem_datain[95:80]   + 16'h0001;
                  mem_datain[79:64]   <= mem_datain[79:64]   + 16'h0001;
                  mem_datain[63:48]   <= mem_datain[63:48]   + 16'h0001;
                  mem_datain[47:32]   <= mem_datain[47:32]   + 16'h0001;
                  mem_datain[31:16]   <= mem_datain[31:16]   + 16'h0001;
                  mem_datain[15:0]    <= mem_datain[15:0]    + 16'h0001;
                    end
                    if(mem_datain[15:0] == mem_dataout[15:0])
                    begin
                      status_reg[0] <= 1'b1;
                    end
                    else
                    begin
                      status_reg[1] <= 1'b1;
                    end
                  end
                end
              end	
        R6:   begin
                if (mem_read_valid)
                  begin
                    if(mem_datain[15:0] == 16'h0040)
                    begin
mem_datain[127:112] <= 16'h0002;
     mem_datain[111:96]  <= 16'h0002;
     mem_datain[95:80]   <= 16'h0002;
     mem_datain[79:64]   <= 16'h0002;
     mem_datain[63:48]   <= 16'h0002;
     mem_datain[47:32]   <= 16'h0002;
     mem_datain[31:16]   <= 16'h0002;
     mem_datain[15:0]    <= 16'h0002;
                      state      <= R7;
                      mem_addr_wen    <= 1'b1;
                    end
                    else
                    begin
                //mem_datain <= mem_datain + 128'h00010001000100010001000100010001;
                  mem_datain[127:112] <= mem_datain[127:112] + 16'h0001;
                  mem_datain[111:96]  <= mem_datain[111:96]  + 16'h0001;
                  mem_datain[95:80]   <= mem_datain[95:80]   + 16'h0001;
                  mem_datain[79:64]   <= mem_datain[79:64]   + 16'h0001;
                  mem_datain[63:48]   <= mem_datain[63:48]   + 16'h0001;
                  mem_datain[47:32]   <= mem_datain[47:32]   + 16'h0001;
                  mem_datain[31:16]   <= mem_datain[31:16]   + 16'h0001;
                  mem_datain[15:0]    <= mem_datain[15:0]    + 16'h0001;
                    end
                    if(mem_datain[15:0] == mem_dataout[15:0])
                    begin
                      status_reg[0] <= 1'b1;
                    end
                    else
                    begin
                      status_reg[1] <= 1'b1;
                    end
                  end
              end
        R7:   begin
                if(mem_addr == (D3 + 32'h007c))
                begin
                  state      <= R8;
                  mem_addr_wen    <= 1'b0;
                  mem_addr   <= D1;
                  if (mem_read_valid)
                  begin
                    if(mem_datain[15:0] == 16'h0041)
                    begin
                      mem_datain[127:112] <= 16'h0003;
     mem_datain[111:96]  <= 16'h0003;
     mem_datain[95:80]   <= 16'h0003;
     mem_datain[79:64]   <= 16'h0003;
     mem_datain[63:48]   <= 16'h0003;
     mem_datain[47:32]   <= 16'h0003;
     mem_datain[31:16]   <= 16'h0003;
     mem_datain[15:0]    <= 16'h0003;
                    end
                    else
                    begin
                //mem_datain <= mem_datain + 128'h00010001000100010001000100010001;
                  mem_datain[127:112] <= mem_datain[127:112] + 16'h0001;
                  mem_datain[111:96]  <= mem_datain[111:96]  + 16'h0001;
                  mem_datain[95:80]   <= mem_datain[95:80]   + 16'h0001;
                  mem_datain[79:64]   <= mem_datain[79:64]   + 16'h0001;
                  mem_datain[63:48]   <= mem_datain[63:48]   + 16'h0001;
                  mem_datain[47:32]   <= mem_datain[47:32]   + 16'h0001;
                  mem_datain[31:16]   <= mem_datain[31:16]   + 16'h0001;
                  mem_datain[15:0]    <= mem_datain[15:0]    + 16'h0001;
                    end
                    if(mem_datain[`raid_width-1:0] == mem_dataout[`raid_width-1:0])
                    begin
                      status_reg[0] <= 1'b1;
                    end
                    else
                    begin
                      status_reg[1] <= 1'b1;
                    end
                  end
                end
                else
                begin
                  mem_addr   <= mem_addr + 3'b100;
                  if (mem_read_valid)
                  begin
                    if(mem_datain == 16'h0041)
                    begin
                      mem_datain[127:112] <= 16'h0003;
     mem_datain[111:96]  <= 16'h0003;
     mem_datain[95:80]   <= 16'h0003;
     mem_datain[79:64]   <= 16'h0003;
     mem_datain[63:48]   <= 16'h0003;
     mem_datain[47:32]   <= 16'h0003;
     mem_datain[31:16]   <= 16'h0003;
     mem_datain[15:0]    <= 16'h0003;
                    end
                    else
                    begin
                //mem_datain <= mem_datain + 128'h00010001000100010001000100010001;
                  mem_datain[127:112] <= mem_datain[127:112] + 16'h0001;
                  mem_datain[111:96]  <= mem_datain[111:96]  + 16'h0001;
                  mem_datain[95:80]   <= mem_datain[95:80]   + 16'h0001;
                  mem_datain[79:64]   <= mem_datain[79:64]   + 16'h0001;
                  mem_datain[63:48]   <= mem_datain[63:48]   + 16'h0001;
                  mem_datain[47:32]   <= mem_datain[47:32]   + 16'h0001;
                  mem_datain[31:16]   <= mem_datain[31:16]   + 16'h0001;
                  mem_datain[15:0]    <= mem_datain[15:0]    + 16'h0001;
                    end
                    if(mem_datain[`raid_width-1:0] == mem_dataout[`raid_width-1:0])
                    begin
                      status_reg[0] <= 1'b1;
                    end
                    else
                    begin
                      status_reg[1] <= 1'b1;
                    end
                  end
                end

              end	
        R8:   begin
                if (mem_read_valid)
                  begin
                    if(mem_datain[15:0] == 16'h0041)
                    begin
                      mem_datain[127:112] <= 16'h0003;
     mem_datain[111:96]  <= 16'h0003;
     mem_datain[95:80]   <= 16'h0003;
     mem_datain[79:64]   <= 16'h0003;
     mem_datain[63:48]   <= 16'h0003;
     mem_datain[47:32]   <= 16'h0003;
     mem_datain[31:16]   <= 16'h0003;
     mem_datain[15:0]    <= 16'h0003;
                      state      <= DONE;
                      mem_addr_wen    <= 1'b0;
                    end
                    else
                    begin
                //mem_datain <= mem_datain + 128'h00010001000100010001000100010001;
                  mem_datain[127:112] <= mem_datain[127:112] + 16'h0001;
                  mem_datain[111:96]  <= mem_datain[111:96]  + 16'h0001;
                  mem_datain[95:80]   <= mem_datain[95:80]   + 16'h0001;
                  mem_datain[79:64]   <= mem_datain[79:64]   + 16'h0001;
                  mem_datain[63:48]   <= mem_datain[63:48]   + 16'h0001;
                  mem_datain[47:32]   <= mem_datain[47:32]   + 16'h0001;
                  mem_datain[31:16]   <= mem_datain[31:16]   + 16'h0001;
                  mem_datain[15:0]    <= mem_datain[15:0]    + 16'h0001;
                    end
                    if(mem_datain[15:0] == mem_dataout[15:0])
                    begin
                      status_reg[0] <= 1'b1;
                    end
                    else
                    begin
                      status_reg[1] <= 1'b1;
                    end
                  end

              end		
        R9:   begin
                 if(mem_addr == (D3 + 32'h007c))
                begin
                  state      <= DONE;
                  mem_addr_wen    <= 1'b0;
                  mem_addr   <= D1;
                  if (mem_read_valid)
                  begin
                    if(mem_datain[15:0] == 16'h0041)
                    begin
                      mem_datain[127:112] <= 16'h0003;
     mem_datain[111:96]  <= 16'h0003;
     mem_datain[95:80]   <= 16'h0003;
     mem_datain[79:64]   <= 16'h0003;
     mem_datain[63:48]   <= 16'h0003;
     mem_datain[47:32]   <= 16'h0003;
     mem_datain[31:16]   <= 16'h0003;
     mem_datain[15:0]    <= 16'h0003;
                    end
                    else
                    begin
                //mem_datain <= mem_datain + 128'h00010001000100010001000100010001;
                  mem_datain[127:112] <= mem_datain[127:112] + 16'h0001;
                  mem_datain[111:96]  <= mem_datain[111:96]  + 16'h0001;
                  mem_datain[95:80]   <= mem_datain[95:80]   + 16'h0001;
                  mem_datain[79:64]   <= mem_datain[79:64]   + 16'h0001;
                  mem_datain[63:48]   <= mem_datain[63:48]   + 16'h0001;
                  mem_datain[47:32]   <= mem_datain[47:32]   + 16'h0001;
                  mem_datain[31:16]   <= mem_datain[31:16]   + 16'h0001;
                  mem_datain[15:0]    <= mem_datain[15:0]    + 16'h0001;
                    end
                    if(mem_datain[15:0] == mem_dataout[15:0])
                    begin
                      status_reg[0] <= 1'b1;
                    end
                    else
                    begin
                      status_reg[1] <= 1'b1;
                    end
                  end
                end
                else
                begin
                  mem_addr   <= mem_addr + 3'b100;
                  if (mem_read_valid)
                  begin
                //mem_datain <= mem_datain + 128'h00010001000100010001000100010001;
                  mem_datain[127:112] <= mem_datain[127:112] + 16'h0001;
                  mem_datain[111:96]  <= mem_datain[111:96]  + 16'h0001;
                  mem_datain[95:80]   <= mem_datain[95:80]   + 16'h0001;
                  mem_datain[79:64]   <= mem_datain[79:64]   + 16'h0001;
                  mem_datain[63:48]   <= mem_datain[63:48]   + 16'h0001;
                  mem_datain[47:32]   <= mem_datain[47:32]   + 16'h0001;
                  mem_datain[31:16]   <= mem_datain[31:16]   + 16'h0001;
                  mem_datain[15:0]    <= mem_datain[15:0]    + 16'h0001;
                    if(mem_datain[15:0] == mem_dataout[15:0])
                    begin
                      status_reg[0] <= 1'b1;
                    end
                    else
                    begin
                      status_reg[1] <= 1'b1;
                    end
                  end
                end
              end	
        R10:   begin
                if (mem_read_valid)
                  begin
                    if(mem_datain[15:0] == 16'h0041)
                    begin
                      mem_datain <= 32'h0000;
                      state      <= R10;
                      mem_addr_wen    <= 1'b0;
                    end
                    else
                    begin
                //mem_datain <= mem_datain + 128'h00010001000100010001000100010001;
                  mem_datain[127:112] <= mem_datain[127:112] + 16'h0001;
                  mem_datain[111:96]  <= mem_datain[111:96]  + 16'h0001;
                  mem_datain[95:80]   <= mem_datain[95:80]   + 16'h0001;
                  mem_datain[79:64]   <= mem_datain[79:64]   + 16'h0001;
                  mem_datain[63:48]   <= mem_datain[63:48]   + 16'h0001;
                  mem_datain[47:32]   <= mem_datain[47:32]   + 16'h0001;
                  mem_datain[31:16]   <= mem_datain[31:16]   + 16'h0001;
                  mem_datain[15:0]    <= mem_datain[15:0]    + 16'h0001;
                    end
                    if(mem_datain[15:0] == mem_dataout[15:0])
                    begin
                      status_reg[0] <= 1'b1;
                    end
                    else
                    begin
                      status_reg[1] <= 1'b1;
                    end
                  end
              end
        R11:   begin
                if(mem_datain[15:0] == 16'h0041)
                begin
                  state      <= DONE;
                  mem_addr_wen    <= 1'b0;
                  
                  if (mem_read_valid)
                  begin
                    if(mem_datain == 128'h0080)
                    begin
                      mem_datain[127:112] <= 16'h0002;
     mem_datain[111:96]  <= 16'h0002;
     mem_datain[95:80]   <= 16'h0002;
     mem_datain[79:64]   <= 16'h0002;
     mem_datain[63:48]   <= 16'h0002;
     mem_datain[47:32]   <= 16'h0002;
     mem_datain[31:16]   <= 16'h0002;
     mem_datain[15:0]    <= 16'h0002;
                    end
                    else
                    begin
                //mem_datain <= mem_datain + 128'h00010001000100010001000100010001;
                  mem_datain[127:112] <= mem_datain[127:112] + 16'h0001;
                  mem_datain[111:96]  <= mem_datain[111:96]  + 16'h0001;
                  mem_datain[95:80]   <= mem_datain[95:80]   + 16'h0001;
                  mem_datain[79:64]   <= mem_datain[79:64]   + 16'h0001;
                  mem_datain[63:48]   <= mem_datain[63:48]   + 16'h0001;
                  mem_datain[47:32]   <= mem_datain[47:32]   + 16'h0001;
                  mem_datain[31:16]   <= mem_datain[31:16]   + 16'h0001;
                  mem_datain[15:0]    <= mem_datain[15:0]    + 16'h0001;
                    end
                    if(mem_datain[15:0] == mem_dataout[15:0])
                    begin
                      status_reg[0] <= 1'b1;
                    end
                    else
                    begin
                      status_reg[1] <= 1'b1;
                    end
                  end
                end
                else
                begin
                  
                  mem_addr   <= mem_addr + 3'b100;
                  if (mem_read_valid)
                  begin
                  
                    if(mem_datain[15:0] == 16'h0041)
                    begin
mem_datain[127:112] <= 16'h0000;
     mem_datain[111:96]  <= 16'h0000;
     mem_datain[95:80]   <= 16'h0000;
     mem_datain[79:64]   <= 16'h0000;
     mem_datain[63:48]   <= 16'h0000;
     mem_datain[47:32]   <= 16'h0000;
     mem_datain[31:16]   <= 16'h0000;
     mem_datain[15:0]    <= 16'h0000;
                    end
                    else
                    begin
                //mem_datain <= mem_datain + 128'h00010001000100010001000100010001;
                  mem_datain[127:112] <= mem_datain[127:112] + 16'h0001;
                  mem_datain[111:96]  <= mem_datain[111:96]  + 16'h0001;
                  mem_datain[95:80]   <= mem_datain[95:80]   + 16'h0001;
                  mem_datain[79:64]   <= mem_datain[79:64]   + 16'h0001;
                  mem_datain[63:48]   <= mem_datain[63:48]   + 16'h0001;
                  mem_datain[47:32]   <= mem_datain[47:32]   + 16'h0001;
                  mem_datain[31:16]   <= mem_datain[31:16]   + 16'h0001;
                  mem_datain[15:0]    <= mem_datain[15:0]    + 16'h0001;
                    end
                    if(mem_datain[15:0] == mem_dataout[15:0])
                    begin
                      status_reg[0] <= 1'b1;
                    end
                    else
                    begin
                      status_reg[1] <= 1'b1;
                    end
                  end
                end
              end	


        DONE: begin
                status_reg[2] <= 1'b1;
              end		  
    endcase
  end
end

endmodule
