//////////////////////////////////////////////////////////////////////////////// 
// Copyright (c) 2007 Xilinx, Inc. 
// All Rights Reserved 
//////////////////////////////////////////////////////////////////////////////// 
//   ____  ____ 
//  /   /\/   / 
// /___/  \  /         Vendor: Xilinx 
// \   \   \/          Version: 1.0
//  \   \              Filename: MUX_BRAM_BLOCK.v 
//  /   /              Date Last Modified: 2/6/2007
// /___/   /\          Date Created: 10/6/2005  
// \   \  /  \ 
//  \___\/\___\ 
// 
//Design Name: MUX_BRAM_BLOCK 
//Purpose:  This module contains a BRAM plus two muxes to direct data in the 
//          RAID IP 
//          
//          This is part of XAPP 731 and XAPP 865
//
////////////////////////////////////////////////////////////////////////////////
// 
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//
////////////////////////////////////////////////////////////////////////////////
`timescale 1ns / 1ps

module MUX_BRAM_BLOCK(
input	 wire  [7:0] bramdatain_a, 
input	 wire  [7:0] bramdatain_b,
input	 wire  [7:0] muxdatain,  
input  wire        clk, 
input  wire        muxsel_a, 
input  wire        muxsel_b,
input  wire [11:0] bram_in_addr, 
input  wire        bram_in_wren, 
input  wire [11:0] bram_out_addr, 
input  wire        bram_out_ren, 
output wire  [7:0] dataout);

wire  [7:0] bramdataout;
wire  [7:0] datain;
wire [23:0] unusedDOB;

//select which data to put in BRAM
assign datain = muxsel_a ? bramdatain_a : bramdatain_b;
//select which data to put output
assign dataout = muxsel_b ? muxdatain : bramdataout;

RAMB16 #(
   .DOA_REG             (1),
   .DOB_REG             (1),
   .INVERT_CLK_DOA_REG  ("FALSE"),
   .INVERT_CLK_DOB_REG  ("FALSE"),
   .READ_WIDTH_A        (9),
   .READ_WIDTH_B        (9),
   .SIM_COLLISION_CHECK ("ALL"),
   .WRITE_MODE_A        ("WRITE_FIRST"),
   .WRITE_MODE_B        ("WRITE_FIRST"),
   .WRITE_WIDTH_A       (9),
   .WRITE_WIDTH_B       (9))
bram0 (
   .CASCADEOUTA (), 
   .CASCADEOUTB (), 
   .DOA         (), 
   .DOB         ({unusedDOB,bramdataout}), 
   .DOPA        (), 
   .DOPB        (), 
   .ADDRA       ({bram_in_addr, 3'b111}), 
   .ADDRB       ({bram_out_addr, 3'b111}), 
   .CASCADEINA  (1'b0), 
   .CASCADEINB  (1'b0), 
   .CLKA        (clk), 
   .CLKB        (clk), 
   .DIA         ({24'hFFF,datain}), 
   .DIB         (32'hFFFF), 
   .DIPA        (), 
   .DIPB        (), 
   .ENA         (bram_in_wren), 
   .ENB         (bram_out_ren), 
   .REGCEA      (1'b1), 
   .REGCEB      (1'b1), 
   .SSRA        (1'b0), 
   .SSRB        (1'b0), 
   .WEA         (4'b1111), 
   .WEB         (4'b0000));

endmodule
