//////////////////////////////////////////////////////////////////////////////// 
// Copyright (c) 2007 Xilinx, Inc. 
// All Rights Reserved 
//////////////////////////////////////////////////////////////////////////////// 
//   ____  ____ 
//  /   /\/   / 
// /___/  \  /         Vendor: Xilinx 
// \   \   \/          Version: 1.1
//  \   \              Filename: RAID_MULT_4.v 
//  /   /              Date Last Modified: 2/05/2007
// /___/   /\          Date Created: 10/05/2005  
// \   \  /  \ 
//  \___\/\___\ 
// 
//Design Name: RAID_MULT_4 
//Purpose:  This module contains four GF math blocks that consist of an 8bit adder 
//          adder plus BRAM that contain the GFILOG and GFLOG LUTs
//          
//          This was part of XAPP 731 and modified for XAPP865
//
////////////////////////////////////////////////////////////////////////////////
// 
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//
////////////////////////////////////////////////////////////////////////////////
`timescale 1ns / 1ps


module RAID_MULT_4(
input	 wire  [7:0] dspin_a, 
input	 wire  [7:0] dspin_b,
input	 wire  [7:0] dspin_c, 
input	 wire  [7:0] dspin_d,
input	 wire  [7:0] gflogin_a, 
input	 wire  [7:0] gflogin_b,
input	 wire  [7:0] gflogin_c, 
input	 wire  [7:0] gflogin_d,
input  wire        gflog_ren_a,
input  wire        gflog_ren_b, 
input  wire        gflog_ren_c,
input  wire        gflog_ren_d,
input  wire        gfilog_ren_a,
input  wire        gfilog_ren_b,
input  wire        gfilog_ren_c,
input  wire        gfilog_ren_d,  
input  wire        clk, 
input  wire        rst,
input  wire        dd_lost,
output wire  [7:0] gfilogout_a, 
output wire  [7:0] gfilogout_b,
output wire  [7:0] gfilogout_c, 
output wire  [7:0] gfilogout_d);

wire  [7:0] gflogdata_a;
wire  [7:0] gflogdata_b;
wire  [7:0] gflogdata_c;
wire  [7:0] gflogdata_d;
wire [11:0]	gflog_a_addr;
wire [11:0] gflog_b_addr;
wire [11:0]	gflog_c_addr;
wire [11:0] gflog_d_addr;
wire [11:0]	gfilog_a_addr;
wire [11:0] gfilog_b_addr;
wire [11:0]	gfilog_c_addr;
wire [11:0] gfilog_d_addr;
wire  [7:0] dspout_a;
wire  [7:0] dspout_b;
wire  [7:0] dspout_c;
wire  [7:0] dspout_d;
wire        gfilog_wea;
wire        gfilog_web;
wire        gfilog_wec;
wire        gfilog_wed;
wire        gflog_wea;
wire        gflog_web;
wire        gflog_wec;
wire        gflog_wed;

wire [15:0] unused_dsp_data;
wire [23:0] unusedDOA;
wire [23:0] unusedDOB;
wire [23:0] unusedDOA2;
wire [23:0] unusedDOB2;
wire [23:0] unusedDOA3;
wire [23:0] unusedDOB3;
wire [23:0] unusedDOA4;
wire [23:0] unusedDOB4;
wire  [7:0] gfilog_d_addr_b ;
reg   [7:0] minus_dsp;

wire  [7:0] pre_gfilogout_a; 
wire  [7:0] pre_gfilogout_b;
wire  [7:0] pre_gfilogout_c; 
wire  [7:0] pre_gfilogout_d;


assign gflog_a_addr = gflogin_a;
assign gflog_b_addr = gflogin_b;
assign gflog_c_addr = gflogin_c;
assign gflog_d_addr = gflogin_d;

assign gfilog_a_addr = dspout_a;
assign gfilog_b_addr = dspout_b;
assign gfilog_c_addr = dspout_c;


assign gfilog_d_addr = minus_dsp;

//this is to take care of cases when the gflog input is zero
reg iszero_a;
reg iszero_a_d;
reg iszero_a_d2;
reg iszero_a_d3;
reg iszero_a_d4;
reg iszero_a_d5;
reg iszero_b;
reg iszero_b_d;
reg iszero_b_d2;
reg iszero_b_d3;
reg iszero_b_d4;
reg iszero_b_d5;
reg iszero_c;
reg iszero_c_d;
reg iszero_c_d2;
reg iszero_c_d3;
reg iszero_c_d4;
reg iszero_c_d5;
reg iszero_d;
reg iszero_d_d;
reg iszero_d_d2;
reg iszero_d_d3;
reg iszero_d_d4;
reg iszero_d_d5;

always @ (posedge clk or posedge rst)
begin
  if (rst)
  begin
    iszero_a    <= 1'b0;
    iszero_a_d  <= 1'b0;
    iszero_a_d2 <= 1'b0;
    iszero_a_d3 <= 1'b0;
    iszero_a_d4 <= 1'b0;
    iszero_a_d5 <= 1'b0;
  end
  else
  begin
    if(gflogin_a == 8'b00000000)
    begin
      iszero_a <= 1'b1;
    end
    else
    begin
      iszero_a  <= 1'b0;
    end
    iszero_a_d  <= iszero_a;
    iszero_a_d2 <= iszero_a_d;
    iszero_a_d3 <= iszero_a_d2;
    iszero_a_d4 <= iszero_a_d3;
    iszero_a_d5 <= iszero_a_d4;
  end
end
always @ (posedge clk or posedge rst)
begin
  if (rst)
  begin
    iszero_b    <= 1'b0;
    iszero_b_d  <= 1'b0;
    iszero_b_d2 <= 1'b0;
    iszero_b_d3 <= 1'b0;
    iszero_b_d4 <= 1'b0;
    iszero_b_d5 <= 1'b0;
  end
  else
  begin
    if(gflogin_b == 8'b00000000)
    begin
      iszero_b <= 1'b1;
    end
    else
    begin
      iszero_b  <= 1'b0;
    end
    iszero_b_d  <= iszero_b;
    iszero_b_d2 <= iszero_b_d;
    iszero_b_d3 <= iszero_b_d2;
    iszero_b_d4 <= iszero_b_d3;
    iszero_b_d5 <= iszero_b_d4;
  end
end
always @ (posedge clk or posedge rst)
begin
  if (rst)
  begin
    iszero_c    <= 1'b0;
    iszero_c_d  <= 1'b0;
    iszero_c_d2 <= 1'b0;
    iszero_c_d3 <= 1'b0;
    iszero_c_d4 <= 1'b0;
    iszero_c_d5 <= 1'b0;
  end
  else
  begin
    if(gflogin_c == 8'b00000000)
    begin
      iszero_c <= 1'b1;
    end
    else
    begin
      iszero_c  <= 1'b0;
    end
    iszero_c_d  <= iszero_c;
    iszero_c_d2 <= iszero_c_d;
    iszero_c_d3 <= iszero_c_d2;
    iszero_c_d4 <= iszero_c_d3;
    iszero_c_d5 <= iszero_c_d4;
  end
end
always @ (posedge clk or posedge rst)
begin
  if (rst)
  begin
    iszero_d    <= 1'b0;
    iszero_d_d  <= 1'b0;
    iszero_d_d2 <= 1'b0;
    iszero_d_d3 <= 1'b0;
    iszero_d_d4 <= 1'b0;
    iszero_d_d5 <= 1'b0;
  end
  else
  begin
    if(gflogin_d == 8'b00000000)
    begin
      iszero_d <= 1'b1;
    end
    else
    begin
      iszero_d  <= 1'b0;
    end
    iszero_d_d  <= iszero_d;
    iszero_d_d2 <= iszero_d_d;
    iszero_d_d3 <= iszero_d_d2;
    iszero_d_d4 <= iszero_d_d3;
    iszero_d_d5 <= iszero_d_d4;
  end
end

assign gfilogout_a = iszero_a_d4 ? 8'b00000000 : pre_gfilogout_a;
assign gfilogout_b = iszero_b_d4 ? 8'b00000000 : pre_gfilogout_b;
assign gfilogout_c = iszero_c_d4 ? 8'b00000000 : pre_gfilogout_c;
assign gfilogout_d = iszero_d_d4 ? 8'b00000000 : pre_gfilogout_d;

//end the special case of zero

always @ (dd_lost or dspout_d or unused_dsp_data[3] )
begin
  if(dd_lost)
  begin
    if (unused_dsp_data[3] )
    begin
      minus_dsp = dspout_d;
    end
    else
    begin
      minus_dsp = (dspout_d - 1'b1);
    end
  end
  else
  begin
    minus_dsp = dspout_d;
  end

end


assign gfilog_wea = gfilog_ren_a;
assign gfilog_web = gfilog_ren_b;
assign gfilog_wec = gfilog_ren_c;
assign gfilog_wed = gfilog_ren_d;
assign gflog_wea = gflog_ren_a;
assign gflog_web = gflog_ren_b;
assign gflog_wec = gflog_ren_c;
assign gflog_wed = gflog_ren_d;


RAMB16 #(
   .DOA_REG             (1),
   .DOB_REG             (1),
   .INVERT_CLK_DOA_REG  ("FALSE"),
   .INVERT_CLK_DOB_REG  ("FALSE"),
   .READ_WIDTH_A        (9),
   .READ_WIDTH_B        (9),
   .SIM_COLLISION_CHECK ("ALL"),
   .WRITE_MODE_A        ("WRITE_FIRST"),
   .WRITE_MODE_B        ("WRITE_FIRST"),
   .WRITE_WIDTH_A       (9),
   .WRITE_WIDTH_B       (9),
   .INIT_00             (256'h714C08C8F869C11C81EF8D340EE064044BC7681BEE33DF03C61A3202190100FF),
   .INIT_01             (256'hA672E44D78099AC9B9F9276A7DC2B51D458212F0DA8E9335210F24E12F658A05),
   .INIT_02             (256'h404638835C13D2F1BDDB968FCE94D03688229110B32598E2FD30DD66628BBF06),
   .INIT_03             (256'h57A7F373ACE5D44E2B79150A9F9B5ECA3DBA85FA54283A6B6E7E48C3A3B6421E),
   .INIT_04             (256'h2E892023D99244117CB4B8267799A5E318FEC531EDDE4A670D63808CF7C07007),
   .INIT_05             (256'hA2416D4753393C849E5D2A14ABD356F261BEFCDCB2978790CDCFBC955BD13F37),
   .INIT_06             (256'h5A3ECCBBB18660FBAA559D29523BA16CF66F0C7FEC4917C476A47BB7D8432D1F),
   .INIT_07             (256'hAF5850A8EAF4D674E8ADE7E6E9D5AE4FD72C757AEB16F50B51A0A99CB05F59CB))
gflogbram0 (
   .CASCADEOUTA (), 
   .CASCADEOUTB (), 
   .DOA         ({unusedDOA,gflogdata_a}), 
   .DOB         ({unusedDOB,gflogdata_b}), 
   .DOPA        (), 
   .DOPB        (), 
   .ADDRA       ({gflog_a_addr, 3'b111}),
   .ADDRB       ({gflog_b_addr, 3'b111}),
   .CASCADEINA  (1'b0), 
   .CASCADEINB  (1'b0), 
   .CLKA        (clk), 
   .CLKB        (clk), 
   .DIA         ({32'hFFFF}), 
   .DIB         ({32'hFFFF}), 
   .DIPA        (), 
   .DIPB        (), 
   .ENA         (1'b1), 
   .ENB         (1'b1), 
   .REGCEA      (1'b1), 
   .REGCEB      (1'b1), 
   .SSRA        (1'b0), 
   .SSRB        (1'b0), 
   .WEA         (4'b0000), 
   .WEB         (4'b0000));

RAMB16 #(
   .DOA_REG             (1),
   .DOB_REG             (1),
   .INVERT_CLK_DOA_REG  ("FALSE"),
   .INVERT_CLK_DOB_REG  ("FALSE"),
   .READ_WIDTH_A        (9),
   .READ_WIDTH_B        (9),
   .SIM_COLLISION_CHECK ("ALL"),
   .WRITE_MODE_A        ("WRITE_FIRST"),
   .WRITE_MODE_B        ("WRITE_FIRST"),
   .WRITE_WIDTH_A       (9),
   .WRITE_WIDTH_B       (9),
   .INIT_00             (256'hC06030180C06038FC9EA75B45A2D984C261387CDE8743A1D8040201008040201),
   .INIT_01             (256'hA1DE6FB9D269BA5DA05028140A058C46239FC1EE77B5D46A35944A259C4E279D),
   .INIT_02             (256'hE271B65BA3DFE1FE7FB1D66BBBD3E7FDF0783C1E0F89CA65BC5E2F99C261BE5F),
   .INIT_03             (256'hCC663397C5EC763B93C7EDF87C3E1F81CE67BDD068341A0D884422118643AFD9),
   .INIT_04             (256'h73B7D5E472399249AA55A452299A4DA8542A158442219E4FA9DA6DB85C2E1785),
   .INIT_05             (256'h41AE57A5DC6E3795C46231964BABDBE3FFF1F67BB3D7E5FC7E3F91C663BFD1E6),
   .INIT_06             (256'h098A45AC562B9BC3EFF9F279B259A251A653A7DDE070381C0E078DC864321982),
   .INIT_07             (256'hFF8E47ADD86C361B83CFE9FA7DB0582C160B8BCBEBFBF3F7F5F47A3D90482412))
gfilogbram0 (
   .CASCADEOUTA (), 
   .CASCADEOUTB (), 
   .DOA         ({unusedDOA2,pre_gfilogout_a}), 
   .DOB         ({unusedDOB2,pre_gfilogout_b}), 
   .DOPA        (), 
   .DOPB        (), 
   .ADDRA       ({gfilog_a_addr, 3'b111}),
   .ADDRB       ({gfilog_b_addr, 3'b111}), 
   .CASCADEINA  (1'b0), 
   .CASCADEINB  (1'b0), 
   .CLKA        (clk), 
   .CLKB        (clk), 
   .DIA         ({32'hFFFF}), 
   .DIB         ({32'hFFFF}), 
   .DIPA        (), 
   .DIPB        (), 
   .ENA         (1'b1), 
   .ENB         (1'b1), 
   .REGCEA      (1'b1), 
   .REGCEB      (1'b1), 
   .SSRA        (1'b0), 
   .SSRB        (1'b0), 
   .WEA         (4'b0000), 
   .WEB         (4'b0000));

RAMB16 #(
   .DOA_REG             (1),
   .DOB_REG             (1),
   .INVERT_CLK_DOA_REG  ("FALSE"),
   .INVERT_CLK_DOB_REG  ("FALSE"),
   .READ_WIDTH_A        (9),
   .READ_WIDTH_B        (9),
   .SIM_COLLISION_CHECK ("ALL"),
   .WRITE_MODE_A        ("WRITE_FIRST"),
   .WRITE_MODE_B        ("WRITE_FIRST"),
   .WRITE_WIDTH_A       (9),
   .WRITE_WIDTH_B       (9),
   .INIT_00             (256'h714C08C8F869C11C81EF8D340EE064044BC7681BEE33DF03C61A3202190100FF),
   .INIT_01             (256'hA672E44D78099AC9B9F9276A7DC2B51D458212F0DA8E9335210F24E12F658A05),
   .INIT_02             (256'h404638835C13D2F1BDDB968FCE94D03688229110B32598E2FD30DD66628BBF06),
   .INIT_03             (256'h57A7F373ACE5D44E2B79150A9F9B5ECA3DBA85FA54283A6B6E7E48C3A3B6421E),
   .INIT_04             (256'h2E892023D99244117CB4B8267799A5E318FEC531EDDE4A670D63808CF7C07007),
   .INIT_05             (256'hA2416D4753393C849E5D2A14ABD356F261BEFCDCB2978790CDCFBC955BD13F37),
   .INIT_06             (256'h5A3ECCBBB18660FBAA559D29523BA16CF66F0C7FEC4917C476A47BB7D8432D1F),
   .INIT_07             (256'hAF5850A8EAF4D674E8ADE7E6E9D5AE4FD72C757AEB16F50B51A0A99CB05F59CB))

gflogbram1 (
   .CASCADEOUTA (), 
   .CASCADEOUTB (), 
   .DOA         ({unusedDOA3,gflogdata_c}), 
   .DOB         ({unusedDOB3,gflogdata_d}), 
   .DOPA        (), 
   .DOPB        (), 
   .ADDRA       ({gflog_c_addr, 3'b111}), 
   .ADDRB       ({gflog_d_addr, 3'b111}), 
   .CASCADEINA  (1'b0), 
   .CASCADEINB  (1'b0), 
   .CLKA        (clk), 
   .CLKB        (clk), 
   .DIA         ({32'hFFFF}), 
   .DIB         ({32'hFFFF}), 
   .DIPA        (), 
   .DIPB        (), 
   .ENA         (1'b1), 
   .ENB         (1'b1), 
   .REGCEA      (1'b1), 
   .REGCEB      (1'b1), 
   .SSRA        (1'b0), 
   .SSRB        (1'b0), 
   .WEA         (4'b0000), 
   .WEB         (4'b0000));

RAMB16 #(
   .DOA_REG             (1),
   .DOB_REG             (1),
   .INVERT_CLK_DOA_REG  ("FALSE"),
   .INVERT_CLK_DOB_REG  ("FALSE"),
   .READ_WIDTH_A        (9),
   .READ_WIDTH_B        (9),
   .SIM_COLLISION_CHECK ("ALL"),
   .WRITE_MODE_A        ("WRITE_FIRST"),
   .WRITE_MODE_B        ("WRITE_FIRST"),
   .WRITE_WIDTH_A       (9),
   .WRITE_WIDTH_B       (9),
   .INIT_00             (256'hC06030180C06038FC9EA75B45A2D984C261387CDE8743A1D8040201008040201),
   .INIT_01             (256'hA1DE6FB9D269BA5DA05028140A058C46239FC1EE77B5D46A35944A259C4E279D),
   .INIT_02             (256'hE271B65BA3DFE1FE7FB1D66BBBD3E7FDF0783C1E0F89CA65BC5E2F99C261BE5F),
   .INIT_03             (256'hCC663397C5EC763B93C7EDF87C3E1F81CE67BDD068341A0D884422118643AFD9),
   .INIT_04             (256'h73B7D5E472399249AA55A452299A4DA8542A158442219E4FA9DA6DB85C2E1785),
   .INIT_05             (256'h41AE57A5DC6E3795C46231964BABDBE3FFF1F67BB3D7E5FC7E3F91C663BFD1E6),
   .INIT_06             (256'h098A45AC562B9BC3EFF9F279B259A251A653A7DDE070381C0E078DC864321982),
   .INIT_07             (256'hFF8E47ADD86C361B83CFE9FA7DB0582C160B8BCBEBFBF3F7F5F47A3D90482412))
gfilogbram1 (
   .CASCADEOUTA (), 
   .CASCADEOUTB (), 
   .DOA         ({unusedDOA4,pre_gfilogout_c}), 
   .DOB         ({unusedDOB4,pre_gfilogout_d}), 
   .DOPA        (), 
   .DOPB        (), 
   .ADDRA       ({gfilog_c_addr, 3'b111}), 
   .ADDRB       ({gfilog_d_addr, 3'b111}), 
   .CASCADEINA  (1'b0), 
   .CASCADEINB  (1'b0), 
   .CLKA        (clk), 
   .CLKB        (clk), 
   .DIA         ({32'hFFFF}), 
   .DIB         ({32'hFFFF}), 
   .DIPA        (), 
   .DIPB        (), 
   .ENA         (1'b1), 
   .ENB         (1'b1), 
   .REGCEA      (1'b1), 
   .REGCEB      (1'b1), 
   .SSRA        (1'b0), 
   .SSRB        (1'b0), 
   .WEA         (4'b0000), 
   .WEB         (4'b0000));


ADDSUB48 addblock_a(
   .CLK        (clk), 
   .RST        (rst), 
   .C_IN       ({12'h000,1'b0,dspin_d,
                       1'b0,dspin_c,
                       1'b0,dspin_b,
                       1'b0,dspin_a}), 
   .PC_IN      ({12'h000,1'b0,gflogdata_d,
                       1'b0,gflogdata_c,
                       1'b0,gflogdata_b,
                       1'b0,gflogdata_a}), 
   .ADD_SUB    (1'b0), //always add
   .ADDSUB_OUT ({unused_dsp_data[15:3],dspout_d,
                  unused_dsp_data[2],dspout_c,
                  unused_dsp_data[1],dspout_b,
                  unused_dsp_data[0],dspout_a}));
	




endmodule
