//////////////////////////////////////////////////////////////////////////////// 
// Copyright (c) 2007 Xilinx, Inc. 
// All Rights Reserved 
//////////////////////////////////////////////////////////////////////////////// 
//   ____  ____ 
//  /   /\/   / 
// /___/  \  /         Vendor: Xilinx 
// \   \   \/          Version: 1.0
//  \   \              Filename: XOR_BRAM.v 
//  /   /              Date Last Modified: 2/28/2007  
// /___/   /\          Date Created: 10/6/2005  
// \   \  /  \ 
//  \___\/\___\ 
// 
//Design Name: XOR_BRAM 
//Purpose:  This module contains a BRAM and an XOR to be used as a building block 
//          for the RAID IP
//          
//          This is part of XAPP 731 and XAPP 865
//
////////////////////////////////////////////////////////////////////////////////
// 
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//
////////////////////////////////////////////////////////////////////////////////
`timescale 1ns / 1ps


module XOR_BRAM(
input	 wire  [7:0] datain, 
input  wire        clk, 
input  wire        rst,
input  wire        muxsel, 
input  wire [11:0] bram_in_addr, 
input  wire        bram_in_wren, 
input  wire [11:0] bram_out_addr, 
input  wire        bram_out_ren, 
output wire  [7:0] dataout);


//internal signals
wire  [7:0] muxed_data;
wire  [7:0] xor_data;
wire [23:0] unusedDOB;

reg  [31:0] datain_reg;

//this is added to allow time for reading the registered
//bram when performing the XOR function
always @(posedge clk or posedge rst)
begin
  if(rst)
  begin
    datain_reg <= 32'h00000000;
  end
  else
  begin
    datain_reg <= datain;
  end
end

//xor data out of BRAM and input data
assign xor_data = dataout ^ datain_reg; 

//select which data to put into the bram
assign muxed_data = muxsel ? xor_data : datain;


RAMB16 #(
   .DOA_REG             (1),
   .DOB_REG             (1),
   .INVERT_CLK_DOA_REG  ("FALSE"),
   .INVERT_CLK_DOB_REG  ("FALSE"),
   .READ_WIDTH_A        (9),
   .READ_WIDTH_B        (9),
   .SIM_COLLISION_CHECK ("ALL"),
   .WRITE_MODE_A        ("WRITE_FIRST"),
   .WRITE_MODE_B        ("WRITE_FIRST"),
   .WRITE_WIDTH_A       (9),
   .WRITE_WIDTH_B       (9))
bram0 (
   .CASCADEOUTA (), 
   .CASCADEOUTB (), 
   .DOA         (), 
   .DOB         ({unusedDOB,dataout}), 
   .DOPA        (), 
   .DOPB        (), 
   .ADDRA       ({bram_in_addr, 3'b111}), 
   .ADDRB       ({bram_out_addr, 3'b111}), 
   .CASCADEINA  (1'b0), 
   .CASCADEINB  (1'b0), 
   .CLKA        (clk), 
   .CLKB        (clk), 
   .DIA         ({24'hFFF,muxed_data}), 
   .DIB         (32'hFFFF), 
   .DIPA        (), 
   .DIPB        (), 
   .ENA         (bram_in_wren), 
   .ENB         (bram_out_ren), 
   .REGCEA      (1'b1), 
   .REGCEB      (1'b1), 
   .SSRA        (1'b0), 
   .SSRB        (1'b0), 
   .WEA         (4'b1111), 
   .WEB         (4'b0000));

endmodule
