///////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2005 Xilinx, Inc.
// This design is confidential and proprietary of Xilinx, All Rights Reserved.
///////////////////////////////////////////////////////////////////////////////
//   ____  ____
//  /   /\/   /
// /___/  \  / Vendor: Xilinx
// \   \   \/ Version: 1.6
//  \   \ Application : MIG
//  /   / Filename: mem_infrastructure.v
// /___/   /\Date Last Modified:  Wed Jun 1 2005
// \   \  /  \Date Created: Mon May 2 2005
//  \___\/\___\
//Device: Virtex-5
//Design Name: DDR2_V5
//Description     :
//
///////////////////////////////////////////////////////////////////////////////

`timescale 1ns/1ps

module mem_infrastructure(

   
   input                     SYS_CLK_N,
   input                     SYS_CLK_P,
   output                    SYS_BUF_CLK,
	input                     CLK200_N,
   input                     CLK200_P,
   input                     SYS_RESET_IN,

   output                    clk_0,
   output                    clk_dv,
   output                    clk_90,
   output                    clk_200,



   
   output reg                sys_rst,
   output reg                sys_rst90,
   output reg                sys_rst200

                      );


   wire       clk_dv_bufg_in;
   wire       clk_dv_bufg_out;

   wire       clk0_bufg_in;
   wire       clk90_bufg_in;
   wire       clk0_bufg_out;
   wire       clk90_bufg_out;
   wire       clk0_bufg1_out;
   wire       REF_CLK200_IN;
   wire       SYS_CLK_IN;
   wire       SYS_RESET;
   wire       dcm_lock;

   reg        sys_rst_0;
   reg        sys_rst_1;
   reg        sys_rst_2;
   reg        sys_rst_3;
   reg        sys_rst_4;
   reg        sys_rst_5;

   reg        sys_rst90_0;
   reg        sys_rst90_1;
   reg        sys_rst90_2;
   reg        sys_rst90_3;
   reg        sys_rst90_4;
   reg        sys_rst90_5;

   reg        sys_rst_200_0;
   reg        sys_rst_200_1;
   reg        sys_rst_200_2;
   reg        sys_rst_200_3;
   reg        sys_rst_200_4;
   reg        sys_rst_200_5;

   assign SYS_BUF_CLK   = SYS_CLK_IN;

   assign clk_dv        = clk_dv_bufg_out;

   assign clk_0         = clk0_bufg_out;
   assign clk_90        = clk90_bufg_out;
	


   assign clk_200       = clk0_bufg1_out;
   assign SYS_RESET   = ~SYS_RESET_IN;

   IBUFGDS_LVPECL_25  lvds_sys_clk_input (
                                          .I   (SYS_CLK_P),
                                          .IB  (SYS_CLK_N),
                                          .O   (SYS_CLK_IN)
                                          );

   IBUFGDS_LVPECL_25 lvpecl_clk200_in (
                                       .O   (REF_CLK200_IN),
                                       .I   (CLK200_P),
                                       .IB  (CLK200_N)
                                       );
 
   defparam DCM_BASE0.DLL_FREQUENCY_MODE = "HIGH";
   defparam DCM_BASE0.DUTY_CYCLE_CORRECTION = "TRUE";
   defparam DCM_BASE0.CLKDV_DIVIDE = 2.0;
   defparam DCM_BASE0.CLKFX_MULTIPLY = 2;
   defparam DCM_BASE0.CLKFX_DIVIDE = 8;
   defparam DCM_BASE0.FACTORY_JF = 16'hF0F0;

   DCM_BASE DCM_BASE0 (
                       .CLK0      (clk0_bufg_in),
                       .CLK180    (),
                       .CLK270    (),
                       .CLK2X     (),
                       .CLK2X180  (),
                       .CLK90     (clk90_bufg_in),
                       .CLKDV     (clk_dv_bufg_in),
                       .CLKFX     (),
                       .CLKFX180  (),
                       .LOCKED    (dcm_lock),
                       .CLKFB     (clk0_bufg_out),
                       .CLKIN     (SYS_CLK_IN),
                       .RST       (SYS_RESET)
                       );





   BUFG dcm_clk0 (
                  .O  (clk0_bufg_out),
                  .I  (clk0_bufg_in)
                  );

   BUFG dcm_clk90 (
                   .O  (clk90_bufg_out),
                   .I  (clk90_bufg_in)
                   );

   BUFG dcm1_clk0 (
                   .O  (clk0_bufg1_out),
                   .I  (REF_CLK200_IN)
                   );

   BUFG dcm1_clk_dv (
                  .O  (clk_dv_bufg_out),
                  .I  (clk_dv_bufg_in)
                  );


   always @ (posedge clk_0)
   begin
      if ( (dcm_lock == 1'b0))
      begin
         sys_rst_0 <= 1'b1;
         sys_rst_1 <= 1'b1;
         sys_rst_2 <= 1'b1;
         sys_rst_3 <= 1'b1;
         sys_rst_4 <= 1'b1;
         sys_rst_5 <= 1'b1;
         sys_rst   <= 1'b1;
      end
      else
      begin
         sys_rst_0 <= 1'b0;
         sys_rst_1 <= sys_rst_0;
         sys_rst_2 <= sys_rst_1;
         sys_rst_3 <= sys_rst_2;
         sys_rst_4 <= sys_rst_3;
         sys_rst_5 <= sys_rst_4;
         sys_rst   <= sys_rst_5;
      end
   end

   always @ (posedge clk_90)
   begin
      if ( (dcm_lock == 1'b0))
      begin
         sys_rst90_0 <= 1'b1;
         sys_rst90_1 <= 1'b1;
         sys_rst90_2 <= 1'b1;
         sys_rst90_3 <= 1'b1;
         sys_rst90_4 <= 1'b1;
         sys_rst90_5 <= 1'b1;
         sys_rst90   <= 1'b1;
      end
      else
      begin
         sys_rst90_0 <= 1'b0;
         sys_rst90_1 <= sys_rst90_0;
         sys_rst90_2 <= sys_rst90_1;
         sys_rst90_3 <= sys_rst90_2;
         sys_rst90_4 <= sys_rst90_3;
         sys_rst90_5 <= sys_rst90_4;
         sys_rst90   <= sys_rst90_5;
      end
   end

   always @ (posedge clk_200)
   begin
      if ((dcm_lock == 1'b0))
      begin
         sys_rst_200_0 <= 1'b1;
         sys_rst_200_1 <= 1'b1;
         sys_rst_200_2 <= 1'b1;
         sys_rst_200_3 <= 1'b1;
         sys_rst_200_4 <= 1'b1;
         sys_rst_200_5 <= 1'b1;
         sys_rst200    <= 1'b1;
      end
      else
      begin
         sys_rst_200_0 <= 1'b0;
         sys_rst_200_1 <= sys_rst_200_0;
         sys_rst_200_2 <= sys_rst_200_1;
         sys_rst_200_3 <= sys_rst_200_2;
         sys_rst_200_4 <= sys_rst_200_3;
         sys_rst_200_5 <= sys_rst_200_4;
         sys_rst200    <= sys_rst_200_5;
      end
   end


endmodule
/*
 //-----------------------------------------------------------------
  //
  //  ICON core wire declarations
  //
  //-----------------------------------------------------------------
  wire [35:0] control0;
  wire [35:0] control1;
  //-----------------------------------------------------------------
  //
  //  ILA Core wire declarations
  //
  //-----------------------------------------------------------------
  wire [31:0] data;
  wire [7:0] trig0;
  wire [31:0] data_big;
  wire [7:0] trig_big0;
  //-----------------------------------------------------------------
  //
  //  VIO Core wire declarations
  //
  //-----------------------------------------------------------------
  //wire [1:0] async_in;
  //wire [3:0] async_out;
  //wire [1:0] sync_in;

assign data_big = { psincdec,psen,psdone};
          


assign trig_big0 = {psincdec,psen,psdone};


//assign async_in = {rst,1'b0};


//assign sync_in = {rst,1'b0};

  //-----------------------------------------------------------------
  //
  //  ICON core instance
  //
  //-----------------------------------------------------------------
  ddr2_icon i_ddr2_icon
    (
      .control0(control0),
      .control1(control1)
    );





 //-----------------------------------------------------------------
  //
  //  ILA BIG core instance
  //
  //-----------------------------------------------------------------
  ila_big i_ila
    (
      .control(control0),
      .clk(SYS_CLK_IN),
      .data(data_big),
      .trig0(trig_big0)
    );


  //-----------------------------------------------------------------
  //
  //  VIO Core wire declarations
  //
  //-----------------------------------------------------------------
  wire [35:0] control;
  wire clk;
  wire [7:0] sync_in;
  wire [7:0] sync_out;
  
  always @(posedge SYS_CLK_IN or posedge SYS_RESET)
  begin
    if(SYS_RESET)
	 begin
	   psen <= 1'b0;
    end
	 else
	 begin
	   if (psen_wire)
		begin
		  psen <= 1'b1;
		end
		else
		begin
		  psen <= 1'b0;
		end
	 end
  end

assign psen_reg = sync_out[0];
assign psincdec = sync_out[1];
assign mem_design_reset = sync_out[2];
assign ps_dcm_reset = sync_out[3];

assign sync_in[0] = psdone;


  //-----------------------------------------------------------------
  //
  //  VIO core instance
  //
  //-----------------------------------------------------------------
  vio_sync i_vio_sync
    (
      .control(control1),
      .clk(SYS_CLK_IN),
      .sync_in(sync_in),
      .sync_out(sync_out)
    );
*/
/*


  //-----------------------------------------------------------------
  //
  //  VIO core instance
  //
  //-----------------------------------------------------------------
  vio i_vio
    (
      .control(control1),
      .clk(clk0),
      .async_in(async_in),
      .async_out(async_out),
      .sync_in(sync_in)
    );

*/
/*

endmodule

//-------------------------------------------------------------------
//
//  ICON core module declaration
//
//-------------------------------------------------------------------
module ddr2_icon 
  (
      control0,
      control1
  );
  output [35:0] control0;
  output [35:0] control1;
endmodule




//-------------------------------------------------------------------
//
//  ILA core module declaration
//
//-------------------------------------------------------------------
module ila
  (
    control,
    clk,
    data,
    trig0
  );
  input [35:0] control;
  input clk;
  input [31:0] data;
  input [7:0] trig0;
endmodule


//-------------------------------------------------------------------
//
//  ILA BIG core module declaration
//
//-------------------------------------------------------------------
module ila_big
  (
    control,
    clk,
    data,
    trig0
  );
  input [35:0] control;
  input clk;
  input [63:0] data;
  input [11:0] trig0;
endmodule





//-------------------------------------------------------------------
//
//  VIO core module declaration
//
//-------------------------------------------------------------------
module vio
  (
    control,
    clk,
    async_in,
    async_out,
    sync_in
  );
  input  [35:0] control;
  input  clk;
  input  [1:0] async_in;
  output [3:0] async_out;
  input  [1:0] sync_in;
endmodule

//-------------------------------------------------------------------
//
//  VIO core module declaration
//
//-------------------------------------------------------------------
module vio_sync
  (
    control,
    clk,
    sync_in,
    sync_out
  );
  input  [35:0] control;
  input  clk;
  input  [7:0] sync_in;
  output [7:0] sync_out;
endmodule
*/
