//////////////////////////////////////////////////////////////////////////////// 
// Copyright (c) 2007 Xilinx, Inc. 
// All Rights Reserved 
//////////////////////////////////////////////////////////////////////////////// 
//   ____  ____ 
//  /   /\/   / 
// /___/  \  /         Vendor: Xilinx 
// \   \   \/          Version: 1.0
//  \   \              Filename: mig_mem_if.v 
//  /   /              Date Last Modified: 2/22/2007
// /___/   /\          Date Created: 10/29/2006  
// \   \  /  \ 
//  \___\/\___\ 
// 
//Design Name: mig_mem_if 
//Purpose:  This module is the logic that interfaces with the xilinx MIG user interface
//          this block can be modified to interface with other type of memory 
//          controller or replaced entirely
//          
//          This is part of Virtex5 version of XAPP 731 to use with DDR2 memory with ECC
//
////////////////////////////////////////////////////////////////////////////////
// 
//     XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
//     SOLELY FOR USE IN DEVELOPING PROGRAMS AND SOLUTIONS FOR
//     XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION
//     AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, APPLICATION
//     OR STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS
//     IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
//     AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
//     FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
//     WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
//     IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
//     REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
//     INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
//     FOR A PARTICULAR PURPOSE.
//
//
////////////////////////////////////////////////////////////////////////////////
`timescale 1ns / 1ps
`include "../rtl/raid6_parameters.v"

module mig_mem_if(
input	 wire        clk,
input  wire        rst,
output wire [`memory_width-1:0] m_wrdata,
output wire  [15:0] m_wr_parity,
input  wire [`memory_width-1:0] m_rddata,
input  wire [15:0] m_rd_parity,
output wire [35:0] m_addr,
output wire        m_data_wen,
output wire        m_addr_wen,
input  wire        m_data_valid,
input  wire        raid_req,
input  wire [31:0] raid_addr,
input  wire        raid_RNW,
input  wire [`memory_width-1:0] raid_wrdata,
input  wire [10:0] sector_len,
output reg         raid_addrack,
output reg         raid_wrrdy,

output reg         raid_dv,
output wire [`memory_width-1:0] raid_rddata,
output wire        raid_complete,
output wire        single_error,
output wire        double_error );

//pipeline logic to pass control and ECC data through the FIFO and on to the memory controller interface
reg  [127:0] raid_wrdata_d1;
reg          m_data_fifo_wen;
reg          m_data_fifo_wen_d1;
reg          m_data_fifo_wen_d2;
reg          m_data_fifo_wen_d3;
reg          m_data_fifo_wen_d4;
reg          m_data_fifo_wen_d5;
reg          m_data_fifo_wen_d6;

reg   [35:0] m_addr_fifo;
reg   [35:0] m_addr_fifo_d1;
reg   [35:0] m_addr_fifo_d2;
reg   [35:0] m_addr_fifo_d3;
reg   [35:0] m_addr_fifo_d4;
reg   [35:0] m_addr_fifo_d5;
reg   [35:0] m_addr_fifo_d6;

reg          m_addr_fifo_wen;
reg          m_addr_fifo_wen_d1;
reg          m_addr_fifo_wen_d2;
reg          m_addr_fifo_wen_d3;
reg          m_addr_fifo_wen_d4;
reg          m_addr_fifo_wen_d5;
reg          m_addr_fifo_wen_d6;

wire [127:0] fifo_rddata;
reg  [127:0] fifo_rddata_d1;
wire  [15:0] fifo_rd_parity;
reg   [15:0] fifo_rd_parity_d1;

assign m_addr     = raid_RNW ? m_addr_fifo : m_addr_fifo_d6;
assign m_addr_wen = raid_RNW ? m_addr_fifo_wen : m_addr_fifo_wen_d6;
assign m_data_wen = m_data_fifo_wen_d6;
assign m_wrdata   = fifo_rddata_d1;
assign m_wr_parity = fifo_rd_parity_d1;


reg   [4:0] state;
reg  [10:0] data_len;
reg   [1:0] write_wait_cnt;
reg         raid_complete_reg;

//for the fifo control logic
reg   [1:0] write_state;
reg   [1:0] read_state;
reg         fifo_ren;
reg         read_complete;
reg         write_complete;
wire        almostfull;
wire        almostempty;
wire        empty;
wire        almostfull_a;
wire        almostempty_a;
wire        empty_a;
wire        almostfull_b;
wire        almostempty_b;
wire        empty_b;

wire        dbiterr_a;
wire        dbiterr_b;
wire        sbiterr_a;
wire        sbiterr_b;

assign single_error = (sbiterr_a  && sbiterr_b);
assign double_error = (dbiterr_a  && dbiterr_b);


assign raid_complete = (write_complete || read_complete);



parameter IDLE  = 5'b00000;
parameter IDLE1 = 5'b00001;
parameter R0    = 5'b00010;
parameter R1    = 5'b00011;
parameter R2    = 5'b00100;
parameter R3    = 5'b00101;
parameter R4    = 5'b00110;
parameter R5    = 5'b00111;
parameter R6    = 5'b01000;
parameter R7    = 5'b01001;
parameter R8    = 5'b01010;
parameter R9    = 5'b01011;
parameter W0    = 5'b10010;
parameter W1    = 5'b10011;
parameter W2    = 5'b10100;
parameter W3    = 5'b10101;
parameter W4    = 5'b10110;
parameter W5    = 5'b10111;
parameter W6    = 5'b11000;
parameter W7    = 5'b11001;
parameter W8    = 5'b11010;
parameter W9    = 5'b11011;
parameter W10   = 5'b11100;


//for the fifo control logic
parameter FIR    = 2'b00;
parameter FR0    = 2'b01;
parameter FR1    = 2'b10;
parameter FR2    = 2'b11;

parameter FIW    = 2'b00;
parameter FW0    = 2'b01;
parameter FW1    = 2'b10;

always @(posedge clk or posedge rst)
begin
  if(rst)
  begin
    m_addr_fifo          <= 36'h000000000;
    raid_wrdata_d1        <= 128'h00000000000000000000000000000000;  
    m_addr_fifo_wen      <= 1'b0;
    m_data_fifo_wen      <= 1'b0;
    raid_addrack    <= 1'b0;
    write_wait_cnt  <= 2'b10;
    raid_wrrdy       <= 1'b0;
    raid_complete_reg    <= 1'b0;
    data_len         <= sector_len;
    state            <= IDLE;

  end
  else 
  begin
    case(state)
      IDLE:  begin
               state          <= IDLE1;
               raid_complete_reg  <= 1'b0;
             end
      IDLE1: begin
               if(raid_req)
               begin
                 if(raid_RNW)
                 begin
                   state <= R0;
                 end
                 else
                 begin
                   state    <= W0;
                   raid_wrrdy      <= 1'b1;
                 end
                 m_addr_fifo[31:0]    <= raid_addr;
                 m_addr_fifo[35:32]   <= ({3'b000,raid_RNW});
                 raid_addrack    <= 1'b1;
                 
               end
               else
               begin
                 state <= IDLE; 
               end   
             end
      R0:    begin
               state         <= R1;
               data_len      <= data_len - 3'b10;
               m_addr_fifo_wen    <= 1'b1;
               m_data_fifo_wen    <= 1'b0;
               raid_addrack  <= 1'b0;
             end
      R1:    begin
               if(data_len == 10'h00)
               begin
                 if(m_data_valid)
                 begin
                   state         <= R3;
                 end
                 else
                 begin
                   state         <= R2;
                 end
                 m_addr_fifo_wen    <= 1'b0;
               end
               else
               begin                
                 state              <= R1;
                 m_addr_fifo_wen    <= 1'b1; 
                 if(m_data_valid)
                 begin
                 end
               end
               data_len           <= data_len - 3'b10;
               m_addr_fifo        <= m_addr_fifo + 3'b100;
             end
      R2:    begin
               if(m_data_valid)
               begin
                 state       <= R3;
               end
               else
               begin
                 state      <= R2;
               end            
             end

      R3:    begin
               if(m_data_valid)
               begin
                 state       <= R3;
               end
               else
               begin
                 state         <= R4;
                 data_len      <= sector_len;
               end
             end
      R4:    begin
               state         <= IDLE;
               raid_complete_reg <= 1'b1;
             end
      W0:    begin
               state         <= W1;
               raid_addrack  <= 1'b0;
             end  
      W1:    begin
               if(write_wait_cnt == 2'b00)
               begin
                 state          <= W2;
                 m_addr_fifo_wen    <= 1'b0;
                 write_wait_cnt <= 2'b10;
                 data_len       <= data_len - 3'b100;
               end
               else
               begin
                 state          <= W1;
                 write_wait_cnt <= write_wait_cnt - 1'b1;
               end
             end       
      W2:    begin
               state         <= W3;
               raid_wrdata_d1      <= raid_wrdata;
               m_addr_fifo_wen    <= 1'b1;
               m_data_fifo_wen    <= 1'b1;   
             end
      W3:    begin
               state         <= W4;
               raid_wrdata_d1      <= raid_wrdata;
               m_addr_fifo_wen    <= 1'b0;
               m_addr_fifo        <= m_addr_fifo + 3'b100;  
             end
      W4:    begin
               state         <= W5;
               m_addr_fifo_wen    <= 1'b1;
               raid_wrdata_d1      <= raid_wrdata;
             end
      W5:    begin
               if(data_len == 10'h00)
               begin
                 state         <= W6;
                 raid_wrdata_d1      <= raid_wrdata;
                 m_addr_fifo_wen    <= 1'b0;
               end
               else
               begin
                 state         <= W2;
                 raid_wrdata_d1      <= raid_wrdata;
                 m_addr_fifo_wen    <= 1'b0;
                 data_len     <= data_len - 3'b100;
                 m_addr_fifo        <= m_addr_fifo + 3'b100;  //for a 4 burst mode
               end
             end
      W6:    begin
               state         <= W7;
               m_data_fifo_wen    <= 1'b0;
               m_addr_fifo_wen    <= 1'b0;
             end
      W7:    begin
               state         <= W8;
               data_len      <= sector_len;
             end
      W8:    begin
               state         <= W9;
               data_len      <= sector_len;
             end
      W9:    begin
               state         <= W10;
               data_len      <= sector_len;
               raid_wrrdy    <= 1'b0; 
               raid_complete_reg <= 1'b1;
             end
     W10:    begin
               state         <= IDLE;
               data_len      <= sector_len;
               raid_complete_reg <= 1'b0;
             end

   default:  begin
               state             <= IDLE;
               m_addr_fifo       <= 32'h00000000;
               raid_wrdata_d1    <= 128'h00000000000000000000000000000000;
               raid_complete_reg <= 1'b0;
               data_len          <= sector_len;
             end
    endcase 
  end
end

//need 2 fifo to consolidate the data on reads since the mig tool has a small ram fifo and caused issues with the burst of the RAID control logic
//these fifos can handle up to 512 but will use the almost full and almost empty to make it 4 less than the raid max and see what that does for performance and latency

//The read FIFO only used the READ portion of the ECC to confirm what was written into the FIFO from the read port of the DDR2 is error free.
//The RAID block doesn't need to use the parity as the data will be modified and the ECC will be regenerated on the WRITE FIFO (see comments 
//for that portion)
//there can be some watchdog logic to keep track when there are ECC errors that occurred

FIFO36_72 #( 
.ALMOST_EMPTY_OFFSET (9'h005),
.ALMOST_FULL_OFFSET  (9'h1C4),
.DO_REG              (1),
.EN_ECC_WRITE        ("FALSE"),
.EN_ECC_READ         ("TRUE"),
.EN_SYN              ("FALSE"),
.FIRST_WORD_FALL_THROUGH ("FALSE"))
fifo_a(
.ALMOSTEMPTY (almostempty_a), 
.ALMOSTFULL  (almostfull_a), 
.DBITERR     (dbiterr_b), 
.DO          (raid_rddata[63:0]), 
.DOP         (), 
.ECCPARITY   (), 
.EMPTY       (empty_a), 
.FULL        (), 
.RDCOUNT     (), 
.RDERR       (), 
.SBITERR     (sbiterr_a), 
.WRCOUNT     (), 
.WRERR       (),	       
.DI          (m_rddata[63:0]), 
.DIP         (m_rd_parity[7:0]), 
.RDCLK       (clk), 
.RDEN        (fifo_ren), 
.RST         (1'b0), 
.WRCLK       (clk), 
.WREN        (m_data_valid)
);

FIFO36_72 #( 
.ALMOST_EMPTY_OFFSET (9'h005),
.ALMOST_FULL_OFFSET  (9'h1C4),
.DO_REG              (1),
.EN_ECC_WRITE        ("FALSE"),
.EN_ECC_READ         ("TRUE"),
.EN_SYN              ("FALSE"),
.FIRST_WORD_FALL_THROUGH ("FALSE"))
fifo_b(
.ALMOSTEMPTY (almostempty_b), 
.ALMOSTFULL  (almostfull_b), 
.DBITERR     (dbiterr_b), 
.DO          (raid_rddata[127:64]), 
.DOP         (), 
.ECCPARITY   (), 
.EMPTY       (empty_b), 
.FULL        (), 
.RDCOUNT     (), 
.RDERR       (), 
.SBITERR     (sbiterr_b), 
.WRCOUNT     (), 
.WRERR       (),	       
.DI          (m_rddata[127:64]), 
.DIP         (m_rd_parity[15:8]), 
.RDCLK       (clk), 
.RDEN        (fifo_ren), 
.RST         (1'b0), 
.WRCLK       (clk), 
.WREN        (m_data_valid)
);


assign almostfull = almostfull_a & almostfull_b;
assign almostempty = almostempty_a & almostempty_b;
assign empty = empty_a & empty_b;


always @(posedge clk or posedge rst)
begin
  if(rst)
  begin
    raid_dv       <= 1'b0;
    read_state    <= FIR;
    fifo_ren      <= 1'b0;
    read_complete <= 1'b0;

  end
  else 
  begin
    case(read_state)

      FIR:  begin
               if(raid_req &&  raid_RNW)
               begin
                 read_state <= FR0;
               end
               else
               begin
                 read_state  <= FIR; 
               end   
             end
       FR0:   begin
               if(almostfull)
               begin
                 fifo_ren      <= 1'b1;
                 read_state   <= FR1; 
               end
               else
               begin
                 read_state  <= FR0; 
               end
             end
       FR1:   begin
               if(empty)
               begin
                 read_state  <= FR2; 
                 fifo_ren      <= 1'b0;
                 read_complete <= 1'b1;
               end
               else
               begin
                 read_state  <= FR1; 
               end
               raid_dv      <= 1'b1;
             end
       FR2:   begin
               read_state  <= FIR; 
               raid_dv      <= 1'b0;
               read_complete <= 1'b0;
             end
   default:  begin
               read_state  <= FIR; 
               raid_dv      <= 1'b0;
               fifo_ren     <= 1'b0;
               read_complete <= 1'b0;
             end
    endcase
  end
end




always @(posedge clk or posedge rst)
begin
  if(rst)
  begin
    m_data_fifo_wen_d1 <= 1'b0;
    m_data_fifo_wen_d2 <= 1'b0;
    m_data_fifo_wen_d3 <= 1'b0;
    m_data_fifo_wen_d4 <= 1'b0;
    m_data_fifo_wen_d5 <= 1'b0;
    m_data_fifo_wen_d6 <= 1'b0;

    m_addr_fifo_d1 <= 36'b0000000000;
    m_addr_fifo_d2 <= 36'b0000000000;
    m_addr_fifo_d3 <= 36'b0000000000;
    m_addr_fifo_d4 <= 36'b0000000000;
    m_addr_fifo_d5 <= 36'b0000000000;
    m_addr_fifo_d6 <= 36'b0000000000;

    m_addr_fifo_wen_d1 <= 1'b0;
    m_addr_fifo_wen_d2 <= 1'b0;
    m_addr_fifo_wen_d3 <= 1'b0;
    m_addr_fifo_wen_d4 <= 1'b0;
    m_addr_fifo_wen_d5 <= 1'b0;
    m_addr_fifo_wen_d6 <= 1'b0;

    fifo_rddata_d1    <= 128'h00000000000000000000000000000000;
    fifo_rd_parity_d1 <= 16'h0000;
  end
  else
  begin
    m_data_fifo_wen_d1 <= m_data_fifo_wen;
    m_data_fifo_wen_d2 <= m_data_fifo_wen_d1;
    m_data_fifo_wen_d3 <= m_data_fifo_wen_d2;
    m_data_fifo_wen_d4 <= m_data_fifo_wen_d3;
    m_data_fifo_wen_d5 <= m_data_fifo_wen_d4;
    m_data_fifo_wen_d6 <= m_data_fifo_wen_d5;

    m_addr_fifo_d1 <= m_addr_fifo;
    m_addr_fifo_d2 <= m_addr_fifo_d1;
    m_addr_fifo_d3 <= m_addr_fifo_d2;
    m_addr_fifo_d4 <= m_addr_fifo_d3;
    m_addr_fifo_d5 <= m_addr_fifo_d4;
    m_addr_fifo_d6 <= m_addr_fifo_d5;

    m_addr_fifo_wen_d1 <= m_addr_fifo_wen;
    m_addr_fifo_wen_d2 <= m_addr_fifo_wen_d1;
    m_addr_fifo_wen_d3 <= m_addr_fifo_wen_d2;
    m_addr_fifo_wen_d4 <= m_addr_fifo_wen_d3;
    m_addr_fifo_wen_d5 <= m_addr_fifo_wen_d4;
    m_addr_fifo_wen_d6 <= m_addr_fifo_wen_d5;

    fifo_rddata_d1    <= fifo_rddata;
    fifo_rd_parity_d1 <= fifo_rd_parity;

  end
end

FIFO36_72 #( 
.ALMOST_EMPTY_OFFSET (9'h005),
.ALMOST_FULL_OFFSET  (9'h1C4),
.DO_REG              (1),
.EN_ECC_WRITE        ("TRUE"),
.EN_ECC_READ         ("FALSE"),
.EN_SYN              ("FALSE"),
.FIRST_WORD_FALL_THROUGH ("FALSE"))
write_fifo_a(
.ALMOSTEMPTY (), 
.ALMOSTFULL  (), 
.DBITERR     (), 
.DO          (fifo_rddata[63:0]), 
.DOP         (fifo_rd_parity[7:0]), 
.ECCPARITY   (), 
.EMPTY       (), 
.FULL        (), 
.RDCOUNT     (), 
.RDERR       (), 
.SBITERR     (), 
.WRCOUNT     (), 
.WRERR       (),	       
.DI          (raid_wrdata_d1[63:0]), 
.DIP         (8'b0), 
.RDCLK       (clk), 
.RDEN        (m_data_fifo_wen_d1 || m_data_fifo_wen_d3|| m_data_fifo_wen_d5), 
.RST         (1'b0), 
.WRCLK       (clk), 
.WREN        (m_data_fifo_wen)
);

FIFO36_72 #( 
.ALMOST_EMPTY_OFFSET (9'h005),
.ALMOST_FULL_OFFSET  (9'h1C4),
.DO_REG              (1),
.EN_ECC_WRITE        ("TRUE"),
.EN_ECC_READ         ("FALSE"),
.EN_SYN              ("FALSE"),
.FIRST_WORD_FALL_THROUGH ("FALSE"))
write_fifo_b(
.ALMOSTEMPTY (), 
.ALMOSTFULL  (), 
.DBITERR     (), 
.DO          (fifo_rddata[127:64]), 
.DOP         (fifo_rd_parity[15:8]), 
.ECCPARITY   (), 
.EMPTY       (), 
.FULL        (), 
.RDCOUNT     (), 
.RDERR       (), 
.SBITERR     (), 
.WRCOUNT     (), 
.WRERR       (),	       
.DI          (raid_wrdata_d1[127:64]), 
.DIP         (8'b0), 
.RDCLK       (clk), 
.RDEN        (m_data_fifo_wen_d1 || m_data_fifo_wen_d3|| m_data_fifo_wen_d5), 
.RST         (1'b0), 
.WRCLK       (clk), 
.WREN        (m_data_fifo_wen)
);




always @(posedge clk or posedge rst)
begin
  if(rst)
  begin
    write_state    <= FIW;
    write_complete <= 1'b0;
  end
  else 
  begin
    case(write_state)

      FIW:  begin
               if(raid_req && ~raid_RNW)
               begin
                   write_state <= FIW;
               end
               else
               begin
                 write_state <= FIW; 
               end   
             end
       FW0:  begin
               if(raid_complete_reg)
               begin
                 write_complete <= 1'b1;
                 write_state   <= FW1;
               end
               else
               begin
                 write_state <= FW0;
               end
             end
       FW1:  begin
               write_complete <= 1'b0;
               write_state <= FIW;
             end
   default:  begin
               write_complete <= 1'b0;
               write_state <= FIW;

             end

    endcase
  end
end




endmodule
